# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2011 Parisson SARL
#
# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>

from django.conf import settings
from django.db.models import Q, Max, Min
from telemeta.models.core import *
from telemeta.util.unaccent import unaccent, unaccent_icmp
from telemeta.models.enum import EthnicGroup
import re

engine = settings.DATABASES['default']['ENGINE']


class MediaItemQuerySet(CoreQuerySet):
    "Base class for all media item query sets"

    def quick_search(self, pattern):
        "Perform a quick search on code, title and collector name"

        # from telemeta.models.media import MediaItem
        # pattern = pattern.strip()
        # mod = MediaItem()
        # fields = mod.to_dict()
        # keys =  fields.keys()
        # q = self.by_fuzzy_collector_q(pattern)
        # for field in keys:
        #     field_str = str(mod._meta.get_field(field))
        #     if 'CharField' in field_str or 'TextField' in field_str:
        #         q = q | word_search_q(field, pattern)

        q = ( Q(code__contains=pattern) |
            Q(old_code__contains=pattern) |
            word_search_q('title', pattern) |
            word_search_q('comment', pattern) |
            self.by_fuzzy_collector_q(pattern) )

        return self.filter(q)


    def without_collection(self):
        "Find items which do not belong to any collection"
        return self.extra(
            where = ["collection_id NOT IN (SELECT id FROM media_collections)"]);

    def by_public_id(self, public_id):
        "Find items by public_id"
        return self.filter(public_id=public_id)

    def by_recording_date(self, from_date, to_date = None):
        "Find items by recording date"
        if to_date is None:
            return (self.filter(recorded_from_date__lte=from_date, recorded_to_date__gte=from_date))
        else :
            return (self.filter(Q(recorded_from_date__range=(from_date, to_date))
                                | Q(recorded_to_date__range=(from_date, to_date))))

    def by_title(self, pattern):
        "Find items by title"
        # to (sort of) sync with models.media.MediaItem.get_title()
        return self.filter(word_search_q("title", pattern) |
                           (Q(title="") & word_search_q("collection__title", pattern)))

    def by_publish_year(self, from_year, to_year = None):
        "Find items by publishing year"
        if to_year is None:
            to_year = from_year
        return self.filter(collection__year_published__range=(from_year, to_year))

    def by_change_time(self, from_time = None, until_time = None):
        "Find items by last change time"
        return self._by_change_time('item', from_time, until_time)

    def by_location(self, location):
        "Find items by location"
        return self.filter(location__in=location.apparented())

    @staticmethod
    def __name_cmp(obj1, obj2):
        return unaccent_icmp(obj1.name, obj2.name)

    def locations(self):
        from telemeta.models import Location, LocationRelation
        l = self.values('location')
        c = self.values('location__current_location')
        r = LocationRelation.objects.filter(location__in=l).values('ancestor_location')
        return Location.objects.filter(Q(pk__in=l) | Q(pk__in=r) | Q(pk__in=c))

    def countries(self, group_by_continent=False):
        countries = []
        from telemeta.models import Location
        for id in self.filter(location__isnull=False).values_list('location', flat=True).distinct():
            location = Location.objects.get(pk=id)
            for l in location.countries():
                c = l.current_location
                if not c in countries:
                    countries.append(c)

        if group_by_continent:
            grouped = {}

            for country in countries:
                for continent in country.continents():
                    if not grouped.has_key(continent):
                        grouped[continent] = []

                    grouped[continent].append(country)

            keys = grouped.keys()
            keys.sort(self.__name_cmp)
            ordered = []
            for c in keys:
                grouped[c].sort(self.__name_cmp)
                ordered.append({'continent': c, 'countries': grouped[c]})

            countries = ordered
        else:
            countries.sort(self.__name_cmp)

        return countries

    def virtual(self, *args):
        qs = self
        need_collection = False
        related = []
        from telemeta.models import Location
        for f in args:
            if f == 'apparent_collector':
                if not 'sqlite3' in engine and not 'postgresql_psycopg2' in engine:
                    related.append('collection')
                    qs = qs.extra(select={f:
                        'IF(collector_from_collection, '
                            'IF(media_collections.collector_is_creator, '
                               'media_collections.creator, '
                               'media_collections.collector),'
                            'media_items.collector)'})
            elif f == 'country_or_continent':
                related.append('location')
                if not 'sqlite3' in engine and not 'postgresql_psycopg2' in engine:
                    qs = qs.extra(select={f:
                        'IF(locations.type = ' + str(Location.COUNTRY) + ' '
                        'OR locations.type = ' + str(Location.CONTINENT) + ','
                        'locations.name, '
                        '(SELECT l2.name FROM location_relations AS r INNER JOIN locations AS l2 '
                        'ON r.ancestor_location_id = l2.id '
                        'WHERE r.location_id = media_items.location_id AND l2.type = ' + str(Location.COUNTRY) + ' LIMIT 1))'
                    })
            else:
                raise Exception("Unsupported virtual field: %s" % f)

        if related:
            qs = qs.select_related(*related)

        return qs

    def ethnic_groups(self):
        ids = self.filter(ethnic_group__isnull=False).values('ethnic_group');
        return EthnicGroup.objects.filter(pk__in=ids)

    @staticmethod
    def by_fuzzy_collector_q(pattern):
        return (word_search_q('collection__creator', pattern) |
                word_search_q('collection__collector', pattern) |
                word_search_q('collector', pattern))

    def by_fuzzy_collector(self, pattern):
        return self.filter(self.by_fuzzy_collector_q(pattern))

    def sound(self):
        return self.filter(Q(file__contains='/') | Q(url__contains='/'))

    def sound_public(self):
        return self.filter(Q(file__contains='/') | Q(url__contains='/'),
                public_access='full', collection__public_access='full')

    def by_instrument(self, name):
        "Find items by instrument"
        from telemeta.models.instrument import Instrument, InstrumentAlias
        from telemeta.models.item import MediaItemPerformance
        instruments = Instrument.objects.filter(name__icontains=name)
        aliases = InstrumentAlias.objects.filter(name__icontains=name)
        perf = []
        performances = MediaItemPerformance.objects.filter(Q(instrument__in=instruments) | Q(alias__in=aliases))
        for performance in performances:
            perf.append(performance)
        return self.filter(performances__in=perf).distinct()

class MediaItemManager(CoreManager):
    "Manage media items queries"

    def get_query_set(self):
        "Return media query sets"
        return MediaItemQuerySet(self.model)

    def enriched(self):
        "Query set with additional virtual fields such as apparent_collector and country_or_continent"
        return self.get_query_set().virtual('apparent_collector', 'country_or_continent')

    def quick_search(self, *args, **kwargs):
        return self.get_query_set().quick_search(*args, **kwargs)
    quick_search.__doc__ = MediaItemQuerySet.quick_search.__doc__

    def without_collection(self, *args, **kwargs):
        return self.get_query_set().without_collection(*args, **kwargs)
    without_collection.__doc__ = MediaItemQuerySet.without_collection.__doc__

    def by_recording_date(self, *args, **kwargs):
        return self.get_query_set().by_recording_date(*args, **kwargs)
    by_recording_date.__doc__ = MediaItemQuerySet.by_recording_date.__doc__

    def by_title(self, *args, **kwargs):
        return self.get_query_set().by_title(*args, **kwargs)
    by_title.__doc__ = MediaItemQuerySet.by_title.__doc__

    def by_publish_year(self, *args, **kwargs):
        return self.get_query_set().by_publish_year(*args, **kwargs)
    by_publish_year.__doc__ = MediaItemQuerySet.by_publish_year.__doc__

    def by_change_time(self, *args, **kwargs):
        return self.get_query_set().by_change_time(*args, **kwargs)
    by_change_time.__doc__ = MediaItemQuerySet.by_change_time.__doc__

    def by_location(self, *args, **kwargs):
        return self.get_query_set().by_location(*args, **kwargs)
    by_location.__doc__ = MediaItemQuerySet.by_location.__doc__

    def sound(self, *args, **kwargs):
        return self.get_query_set().sound(*args, **kwargs)
    sound.__doc__ = MediaItemQuerySet.sound.__doc__

    def sound_public(self, *args, **kwargs):
        return self.get_query_set().sound_public(*args, **kwargs)
    sound_public.__doc__ = MediaItemQuerySet.sound_public.__doc__

    def by_instrument(self, *args, **kwargs):
        return self.get_query_set().by_instrument(*args, **kwargs)
    by_instrument.__doc__ = MediaItemQuerySet.by_instrument.__doc__


class MediaCollectionQuerySet(CoreQuerySet):

    def quick_search(self, pattern):
        "Perform a quick search on code, title and collector name"
        from telemeta.models.collection import MediaCollection
        pattern = pattern.strip()
        mod = MediaCollection()
        fields = mod.to_dict()
        keys =  fields.keys()
        q = self.by_fuzzy_collector_q(pattern)
        for field in keys:
            field_str = str(mod._meta.get_field(field))
            if 'CharField' in field_str or 'TextField' in field_str:
                q = q | word_search_q(field, pattern)
        return self.filter(q)

    def by_location(self, location):
        "Find collections by location"
        return self.filter(items__location__in=location.apparented()).distinct()

    def by_recording_year(self, from_year, to_year=None):
        "Find collections by recording year"
        if to_year is None:
            return (self.filter(recorded_from_year__lte=from_year, recorded_to_year__gte=from_year))
        else:
            return (self.filter(Q(recorded_from_year__range=(from_year, to_year)) |
                    Q(recorded_to_year__range=(from_year, to_year))))

    def by_publish_year(self, from_year, to_year=None):
        "Find collections by publishing year"
        if to_year is None:
            to_year = from_year
        return self.filter(year_published__range=(from_year, to_year))

    def by_ethnic_group(self, group):
        "Find collections by ethnic group"
        return self.filter(items__ethnic_group=group).distinct()

    def by_change_time(self, from_time=None, until_time=None):
        "Find collections between two dates"
        return self._by_change_time('collection', from_time, until_time)

    def virtual(self, *args):
        qs = self
        for f in args:
            if f == 'apparent_collector':
                if not 'sqlite3' in engine and not 'postgresql_psycopg2' in engine:
                    qs = qs.extra(select={f: 'IF(media_collections.collector_is_creator, '
                                         'media_collections.creator, media_collections.collector)'})
            else:
                raise Exception("Unsupported virtual field: %s" % f)

        return qs

    def recording_year_range(self):
        from_max = self.aggregate(Max('recorded_from_year'))['recorded_from_year__max']
        to_max   = self.aggregate(Max('recorded_to_year'))['recorded_to_year__max']
        year_max = max(from_max, to_max)

        from_min = self.filter(recorded_from_year__gt=0).aggregate(Min('recorded_from_year'))['recorded_from_year__min']
        to_min   = self.filter(recorded_to_year__gt=0).aggregate(Min('recorded_to_year'))['recorded_to_year__min']
        year_min = min(from_min, to_min)

        if not year_max:
            year_max = year_min
        elif not year_min:
            year_min = year_max

        return year_min, year_max

    def publishing_year_range(self):
        year_max = self.aggregate(Max('year_published'))['year_published__max']
        year_min = self.filter(year_published__gt=0).aggregate(Min('year_published'))['year_published__min']

        return year_min, year_max

    @staticmethod
    def by_fuzzy_collector_q(pattern):
        return word_search_q('creator', pattern) | word_search_q('collector', pattern)

    def by_fuzzy_collector(self, pattern):
        return self.filter(self.by_fuzzy_collector_q(pattern))

    def sound(self):
        return self.filter(Q(items__file__contains='/') | Q(items__url__contains='/')).distinct()

    def by_instrument(self, name):
        "Find collections by instrument"
        from telemeta.models.item import MediaItemPerformance
        from telemeta.models.instrument import Instrument, InstrumentAlias
        instruments = Instrument.objects.filter(name__icontains=name)
        aliases = InstrumentAlias.objects.filter(name__icontains=name)
        items = []
        performances = MediaItemPerformance.objects.filter(Q(instrument__in=instruments) | Q(alias__in=aliases))
        for performance in performances:
            items.append(performance.media_item)
        return self.filter(items__in=items).distinct()


class MediaCollectionManager(CoreManager):
    "Manage collection queries"

    def get_query_set(self):
        "Return the collection query"
        return MediaCollectionQuerySet(self.model)

    def enriched(self):
        "Query set with additional virtual fields such as apparent_collector"
        return self.get_query_set().virtual('apparent_collector')

    def quick_search(self, *args, **kwargs):
        return self.get_query_set().quick_search(*args, **kwargs)
    quick_search.__doc__ = MediaCollectionQuerySet.quick_search.__doc__

    def by_location(self, *args, **kwargs):
        return self.get_query_set().by_location(*args, **kwargs)
    by_location.__doc__ = MediaCollectionQuerySet.by_location.__doc__

    def by_recording_year(self, *args, **kwargs):
        return self.get_query_set().by_recording_year(*args, **kwargs)
    by_recording_year.__doc__ = MediaCollectionQuerySet.by_recording_year.__doc__

    def by_publish_year(self, *args, **kwargs):
        return self.get_query_set().by_publish_year(*args, **kwargs)
    by_publish_year.__doc__ = MediaCollectionQuerySet.by_publish_year.__doc__

    def by_ethnic_group(self, *args, **kwargs):
        return self.get_query_set().by_ethnic_group(*args, **kwargs)
    by_ethnic_group.__doc__ = MediaCollectionQuerySet.by_ethnic_group.__doc__

    def by_change_time(self, *args, **kwargs):
        return self.get_query_set().by_change_time(*args, **kwargs)
    by_change_time.__doc__ = MediaCollectionQuerySet.by_change_time.__doc__

    @staticmethod
    def __name_cmp(obj1, obj2):
        return unaccent_icmp(obj1.name, obj2.name)

    def sound(self, *args, **kwargs):
        return self.get_query_set().sound(*args, **kwargs)
    sound.__doc__ = MediaCollectionQuerySet.sound.__doc__

    def by_instrument(self, *args, **kwargs):
        return self.get_query_set().by_instrument(*args, **kwargs)
    by_instrument.__doc__ = MediaCollectionQuerySet.by_instrument.__doc__


class LocationQuerySet(CoreQuerySet):
    __flatname_map = None

    def by_flatname(self, flatname):
        map = self.flatname_map()
        return self.filter(pk=map[flatname])

    def flatname_map(self):
        if self.__class__.__flatname_map:
            return self.__class__.__flatname_map

        map = {}
        locations = self.filter(Q(type=self.model.COUNTRY) | Q(type=self.model.CONTINENT))
        for l in locations:
            flatname = unaccent(l.name).lower()
            flatname = re.sub('[^a-z]', '_', flatname)
            while map.has_key(flatname):
                flatname = '_' + flatname
            map[flatname] = l.id

        self.__class__.__flatname_map = map
        return map

    def current(self):
        return self.filter(id__in=self.values_list('current_location_id', flat=True)).distinct()


class LocationManager(CoreManager):

    def get_query_set(self):
        "Return location query set"
        return LocationQuerySet(self.model)

    def by_flatname(self, *args, **kwargs):
        return self.get_query_set().by_flatname(*args, **kwargs)
    by_flatname.__doc__ = LocationQuerySet.by_flatname.__doc__

    def flatname_map(self, *args, **kwargs):
        return self.get_query_set().flatname_map(*args, **kwargs)
    flatname_map.__doc__ = LocationQuerySet.flatname_map.__doc__


class MediaCorpusQuerySet(CoreQuerySet):
    "Base class for all media resource query sets"

    def quick_search(self, pattern):
        "Perform a quick search on text and char fields"
        from telemeta.models.corpus import MediaCorpus
        mod = MediaCorpus()
        pattern = pattern.strip()
        q = Q(code__contains=pattern)
        fields = mod.to_dict()
        keys =  fields.keys()

        for field in keys:
            field_str = str(mod._meta.get_field(field))
            if 'CharField' in field_str or 'TextField' in field_str:
                q = q | word_search_q(field, pattern)

        return self.filter(q)


class MediaCorpusManager(CoreManager):
    "Manage media resource queries"

    def get_query_set(self):
        "Return resource query sets"
        return MediaCorpusQuerySet(self.model)

    def quick_search(self, *args, **kwargs):
        return self.get_query_set().quick_search(*args, **kwargs)
    quick_search.__doc__ = MediaCorpusQuerySet.quick_search.__doc__


class MediaFondsQuerySet(CoreQuerySet):
    "Base class for all media resource query sets"

    def quick_search(self, pattern):
        "Perform a quick search on text and char fields"
        from telemeta.models.fonds import MediaFonds
        mod = MediaFonds()
        pattern = pattern.strip()
        q = Q(code__contains=pattern)
        fields = mod.to_dict()
        keys =  fields.keys()
        for field in keys:
            field_str = str(mod._meta.get_field(field))
            if 'CharField' in field_str or 'TextField' in field_str:
                q = q | word_search_q(field, pattern)
        return self.filter(q)


class MediaFondsManager(CoreManager):
    "Manage media resource queries"

    def get_query_set(self):
        "Return resource query sets"
        return MediaFondsQuerySet(self.model)

    def quick_search(self, *args, **kwargs):
        return self.get_query_set().quick_search(*args, **kwargs)
    quick_search.__doc__ = MediaFondsQuerySet.quick_search.__doc__


class InstrumentQuerySet(CoreQuerySet):
    "Base class for all media instrument query sets"

    def quick_search(self, pattern):
        "Perform a quick search on text and char fields"
        from telemeta.models.instrument import Instrument
        mod = Instrument()
        pattern = pattern.strip()
        q = Q(code__contains=pattern)
        fields = mod.to_dict()
        keys =  fields.keys()
        for field in keys:
            field_str = str(mod._meta.get_field(field))
            if 'CharField' in field_str or 'TextField' in field_str:
                q = q | word_search_q(field, pattern)
        return self.filter(q)


class InstrumentManager(CoreManager):
    "Manage instrument queries"

    def get_query_set(self):
        "Return instrument query sets"
        return InstrumentQuerySet(self.model)

    def quick_search(self, *args, **kwargs):
        return self.get_query_set().quick_search(*args, **kwargs)
    quick_search.__doc__ = InstrumentQuerySet.quick_search.__doc__



# -*- coding: utf-8 -*-
# Copyright (C) 2011-2014 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Guillaume Pellerin <yomguy@parisson.com>

import django.forms as forms
from django.forms import ModelForm
from django.contrib.admin.widgets import FilteredSelectMultiple
from telemeta.models import *
from extra_views import CreateWithInlinesView, UpdateWithInlinesView, InlineFormSet
from extra_views.generic import GenericInlineFormSet
from django.forms.widgets import HiddenInput
from django.utils.translation import ugettext_lazy as _

class MediaFondsForm(ModelForm):

    queryset = MediaCorpus.objects.all()
    widget = FilteredSelectMultiple("Corpus", True,)
    children = forms.ModelMultipleChoiceField(widget=widget, queryset=queryset, label='Corpus')

    class Meta:
        model = MediaFonds
        exclude = ['description', 'public_access']

    class Media:
        css = {'all': ['/static/admin/css/widgets.css',],}
        js = ['/admin/django/jsi18n/',]


class MediaCorpusForm(ModelForm):

    queryset = MediaCollection.objects.all()
    widget = FilteredSelectMultiple('Collections', False)
    children = forms.ModelMultipleChoiceField(widget=widget, queryset=queryset,label='Collections')

    class Meta:
        model = MediaCorpus
        exclude = ['description', 'public_access']

    class Media:
        css = {'all': ('/static/admin/css/widgets.css',),}
        js = ('/admin/django/jsi18n/',)


class MediaCollectionForm(ModelForm):

    def __init__(self, *args, **kwargs):
        super(MediaCollectionForm, self).__init__(*args, **kwargs)
        if '_I_' in self.instance.code:
            self.fields["reference"].widget = HiddenInput()
        if self.instance.computed_duration:
            self.fields["approx_duration"].widget = HiddenInput()

    class Meta:
        model = MediaCollection
        exclude = model.exclude

    def clean_doctype_code(self):
        return self.cleaned_data['doctype_code'] or 0


class MediaItemForm(ModelForm):

    class Meta:
        model = MediaItem
        exclude = model.exclude

    def clean_code(self):
        return self.cleaned_data['code'] or None


class RestrictedMediaItemForm(MediaItemForm):

    class Meta:
        model = MediaItem
        exclude = model.restricted


class PlaylistForm(ModelForm):

    class Meta:
        model = Playlist


class FondsRelatedInline(InlineFormSet):

    model = MediaFondsRelated
    exclude = ['mime_type']


class CorpusRelatedInline(InlineFormSet):

    model = MediaCorpusRelated
    exclude = ['mime_type']


class CollectionRelatedInline(InlineFormSet):

    model = MediaCollectionRelated
    exclude = ['mime_type']


class ItemRelatedInline(InlineFormSet):

    model = MediaItemRelated
    exclude = ['mime_type']


class CollectionIdentifierInline(InlineFormSet):

    model = MediaCollectionIdentifier
    max_num = 1


class ItemPerformanceInline(InlineFormSet):

    model = MediaItemPerformance


class ItemKeywordInline(InlineFormSet):

    model = MediaItemKeyword


class ItemFormatInline(InlineFormSet):

    model = Format
    max_num = 1


class ItemIdentifierInline(InlineFormSet):

    model = MediaItemIdentifier
    max_num = 1


class EpubPasswordForm(forms.Form):

    password = forms.CharField(label=_('password'))



# This file by Fredrik Lundh from:
# http://effbot.org/zone/unicode-convert.htm
# http://effbot.python-hosting.com/file/stuff/sandbox/text/unaccent.py

# use a dynamically populated translation dictionary to remove accents
# from a string

import unicodedata, sys

CHAR_REPLACEMENT = {
    # latin-1 characters that don't have a unicode decomposition
    0xc6: u"AE", # LATIN CAPITAL LETTER AE
    0xd0: u"D",  # LATIN CAPITAL LETTER ETH
    0xd8: u"OE", # LATIN CAPITAL LETTER O WITH STROKE
    0xde: u"Th", # LATIN CAPITAL LETTER THORN
    0xdf: u"ss", # LATIN SMALL LETTER SHARP S
    0xe6: u"ae", # LATIN SMALL LETTER AE
    0xf0: u"d",  # LATIN SMALL LETTER ETH
    0xf8: u"oe", # LATIN SMALL LETTER O WITH STROKE
    0xfe: u"th", # LATIN SMALL LETTER THORN
    }

##
# Translation dictionary.  Translation entries are added to this
# dictionary as needed.

class UnaccentedMap(dict):

    ##
    # Maps a unicode character code (the key) to a replacement code
    # (either a character code or a unicode string).

    def mapchar(self, key):
        ch = self.get(key)
        if ch is not None:
            return ch
        de = unicodedata.decomposition(unichr(key))
        if de:
            try:
                ch = int(de.split(None, 1)[0], 16)
            except (IndexError, ValueError):
                ch = key
        else:
            ch = CHAR_REPLACEMENT.get(key, key)
        self[key] = ch
        return ch

    if sys.version >= "2.5":
        # use __missing__ where available
        __missing__ = mapchar
    else:
        # otherwise, use standard __getitem__ hook (this is slower,
        # since it's called for each character)
        __getitem__ = mapchar


_map = UnaccentedMap()

def unaccent(str):
    return str.translate(_map)

def unaccent_icmp(str1, str2):
    str1 = unaccent(str1).lower()
    str2 = unaccent(str2).lower()
    if str1 > str2:
        return 1

    if str1 < str2:
        return -1

    return 0        



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *
from telemeta.views.epub import *

class CollectionView(object):
    """Provide Collections web UI methods"""

    def collection_detail(self, request, public_id, template='telemeta/collection_detail.html'):
        collection = MediaCollection.objects.get(public_id=public_id)
        items = collection.items.enriched()
        items = items.order_by('code', 'old_code')

        if collection.public_access == 'none' and not (request.user.is_staff or request.user.is_superuser):
            mess = ugettext('Access not allowed')
            title = ugettext('Collection') + ' : ' + public_id + ' : ' + mess
            description = ugettext('Please login or contact the website administator to get a private access.')
            messages.error(request, title)
            return render(request, 'telemeta/messages.html', {'description' : description})

        playlists = get_playlists_names(request)

        related_media = MediaCollectionRelated.objects.filter(collection=collection)
        check_related_media(related_media)
        parents = MediaCorpus.objects.filter(children=collection)
        revisions = Revision.objects.filter(element_type='collection',
                                            element_id=collection.id).order_by('-time')
        if revisions:
            last_revision = revisions[0]
        else:
            last_revision = None

        return render(request, template, {'collection': collection, 'playlists': playlists,
                'items': items, 'related_media': related_media,
                'parents': parents, 'last_revision': last_revision })

    @method_decorator(permission_required('telemeta.change_mediacollection'))
    def collection_edit(self, request, public_id, template='telemeta/collection_edit.html'):
        collection = MediaCollection.objects.get(public_id=public_id)
        if request.method == 'POST':
            form = MediaCollectionForm(data=request.POST, files=request.FILES, instance=collection)
            if form.is_valid():
                code = form.cleaned_data['code']
                if not code:
                    code = public_id
                form.save()
                collection.set_revision(request.user)
                return redirect('telemeta-collection-detail', code)
        else:
            form = MediaCollectionForm(instance=collection)

        return render(request, template, {'collection': collection, "form": form,})

    @method_decorator(permission_required('telemeta.add_mediacollection'))
    def collection_add(self, request, template='telemeta/collection_add.html'):
        collection = MediaCollection()
        if request.method == 'POST':
            form = MediaCollectionForm(data=request.POST, files=request.FILES, instance=collection)
            if form.is_valid():
                code = form.cleaned_data['code']
                if not code:
                    code = public_id
                form.save()
                collection.set_revision(request.user)
                return redirect('telemeta-collection-detail', code)
        else:
            form = MediaCollectionForm(instance=collection)

        return render(request, template, {'collection': collection, "form": form,})

    @method_decorator(permission_required('telemeta.add_mediacollection'))
    def collection_copy(self, request, public_id, template='telemeta/collection_edit.html'):
        if request.method == 'POST':
            collection = MediaCollection()
            form = MediaCollectionForm(data=request.POST, files=request.FILES, instance=collection)
            if form.is_valid():
                code = form.cleaned_data['code']
                if not code:
                    code = public_id
                form.save()
                collection.set_revision(request.user)
                return redirect('telemeta-collection-detail', code)
        else:
            collection = MediaCollection.objects.get(public_id=public_id)
            form = MediaCollectionForm(instance=collection)

        return render(request, template, {'collection': collection, "form": form,})

    def collection_playlist(self, request, public_id, template, mimetype):
        try:
            collection = MediaCollection.objects.get(public_id=public_id)
        except ObjectDoesNotExist:
            raise Http404

        template = loader.get_template(template)
        context = RequestContext(request, {'collection': collection, 'host': request.META['HTTP_HOST']})
        return HttpResponse(template.render(context), content_type=mimetype)

    @method_decorator(permission_required('telemeta.delete_mediacollection'))
    def collection_delete(self, request, public_id):
        """Delete a given collection"""
        collection = MediaCollection.objects.get(public_id=public_id)
        revisions = Revision.objects.filter(element_type='collection', element_id=collection.id)
        for revision in revisions:
            revision.delete()
        collection.delete()
        return redirect('telemeta-collections')

    def related_media_collection_stream(self, request, public_id, media_id):
        collection = MediaCollection.objects.get(public_id=public_id)
        media = MediaCollectionRelated.objects.get(collection=collection, id=media_id)
        response = StreamingHttpResponse(stream_from_file(media.file.path), content_type=media.mime_type)
#        response['Content-Disposition'] = 'attachment'
        return response

    def related_media_collection_download(self, request, public_id, media_id):
        collection = MediaCollection.objects.get(public_id=public_id)
        media = MediaCollectionRelated.objects.get(collection=collection, id=media_id)
        filename = media.file.path.split(os.sep)[-1]
        response = StreamingHttpResponse(stream_from_file(media.file.path), content_type=media.mime_type)
        response['Content-Disposition'] = 'attachment; ' + 'filename=' + filename
        return response

    @method_decorator(permission_required('telemeta.change_mediacollection'))
    def related_media_edit(self, request, public_id, template):
        collection = MediaCollection.objects.get(public_id=public_id)
        MediaCollectionRelatedFormSet = inlineformset_factory(MediaCollection, MediaCollectionRelated, form=MediaCollectionRelatedForm)
        if request.method == 'POST':
            formset = MediaCollectionRelatedFormSet(data=request.POST, files=request.FILES, instance=collection)
            if formset.is_valid():
                formset.save()
                collection.set_revision(request.user)
                return redirect('telemeta-collection-edit', public_id)
        else:
            formset = MediaCollectionRelatedFormSet(instance=collection)

        return render(request, template, {'collection': collection, 'formset': formset,})


class CollectionZipView(View):

    model = MediaCollection

    def get_object(self):
        return MediaCollection.objects.get(public_id=self.kwargs['public_id'])

    def get(self, request, *args, **kwargs):
        """
        Stream a ZIP file of collection data
        without loading the whole file into memory.
        Based on ZipStream
        """
        from telemeta.views import MarkerView
        from telemeta.backup import CollectionSerializer
        import zipstream
        from zipfile import ZIP_DEFLATED, ZIP_STORED
        import json

        zip_file = zipstream.ZipFile(mode='w', compression=ZIP_STORED,
                                     allowZip64=True)
        cache_data = TelemetaCache(settings.TELEMETA_DATA_CACHE_DIR)

        collection = self.get_object()
        serializer = CollectionSerializer(collection)

        data = collection.get_json().encode('utf-8')
        filename = collection.public_id + '.json'
        cache_data.write_bin(data, filename)
        path = cache_data.dir + os.sep + filename
        zip_file.write(path, arcname=collection.public_id + os.sep + filename)

        data = serializer.get_xml().encode('utf-8')
        filename = collection.public_id + '.xml'
        cache_data.write_bin(data, filename)
        path = cache_data.dir + os.sep + filename
        zip_file.write(path, arcname=collection.public_id + os.sep + filename)

        for item in collection.items.all():
            if item.file:
                filename, ext = os.path.splitext(item.file.path.split(os.sep)[-1])
                zip_file.write(item.file.path, arcname=collection.public_id + os.sep + item.code + ext)
            marker_view = MarkerView()
            markers = marker_view.get_markers(item.id)
            if markers:
                data = json.dumps(markers)
                filename = item.code + '.json'
                cache_data.write_bin(data, filename)
                path = cache_data.dir + os.sep + filename
                zip_file.write(path, arcname=collection.public_id + os.sep + filename)

        response = StreamingHttpResponse(zip_file, content_type='application/zip')
        response['Content-Disposition'] = "attachment; filename=%s.%s" % \
                                             (collection.code, 'zip')
        return response

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(CollectionZipView, self).dispatch(*args, **kwargs)


class CollectionViewMixin(object):

    model = MediaCollection
    form_class = MediaCollectionForm

    def get_object(self):
        obj = self.model.objects.filter(code=self.kwargs['public_id'])
        if not obj:
            if self.kwargs['public_id'].isdigit():
                try:
                    obj = self.model.objects.get(id=self.kwargs['public_id'])
                except self.model.DoesNotExist:
                    raise Http404
            else:
                raise Http404
        else:
            obj = obj[0]
        return obj


class CollectionListView(ListView):

    model = MediaCollection
    template_name = "telemeta/collection_list.html"
    paginate_by = 20
    queryset = MediaCollection.objects.enriched()

    def get_context_data(self, **kwargs):
        context = super(CollectionListView, self).get_context_data(**kwargs)
        context['count'] = self.object_list.count()
        return context


class CollectionUnpublishedListView(CollectionListView):

    queryset = MediaCollection.objects.filter(code__contains='_I_')


class CollectionPublishedListView(CollectionListView):

    queryset = MediaCollection.objects.filter(code__contains='_E_')


class CollectionSoundListView(CollectionListView):

    queryset = MediaCollection.objects.sound().order_by('code', 'old_code')


class CollectionDetailView(CollectionViewMixin, DetailView):

    template_name = 'telemeta/collection_detail.html'

    def get_context_data(self, **kwargs):
        context = super(CollectionDetailView, self).get_context_data(**kwargs)
        collection = self.get_object()
        items = collection.items.enriched()
        context['collection'] = collection
        context['items'] = items.order_by('code', 'old_code')
        context['playlists'] = get_playlists_names(self.request)
        context['related_media'] = MediaCollectionRelated.objects.filter(collection=collection)
        check_related_media(context['related_media'])
        context['parents'] = MediaCorpus.objects.filter(children=collection)
        revisions = Revision.objects.filter(element_type='collection',
                                            element_id=collection.id)
        if revisions:
            context['last_revision'] = revisions[0]
        else:
            context['last_revision'] = None

        return context


class CollectionDetailViewDC(CollectionDetailView):

    template_name = "telemeta/collection_detail_dc.html"


class CollectionEditView(CollectionViewMixin, UpdateWithInlinesView):

    template_name = 'telemeta/collection_edit.html'
    inlines = [CollectionRelatedInline, CollectionIdentifierInline]

    def forms_valid(self, form, inlines):
        messages.info(self.request, ugettext_lazy("You have successfully updated your collection."))
        obj = form.save()
        obj.set_revision(self.request.user)
        return super(CollectionEditView, self).forms_valid(form, inlines)

    def get_success_url(self):
        return reverse_lazy('telemeta-collection-detail', kwargs={'public_id':self.kwargs['public_id']})

    def get_context_data(self, **kwargs):
        context = super(CollectionEditView, self).get_context_data(**kwargs)
        collection = self.get_object()
        context['collection'] = collection
        return context

    @method_decorator(permission_required('telemeta.change_mediacollection'))
    def dispatch(self, *args, **kwargs):
        return super(CollectionEditView, self).dispatch(*args, **kwargs)


class CollectionAddView(CollectionViewMixin, CreateWithInlinesView):

    template_name = 'telemeta/collection_add.html'
    inlines = [CollectionRelatedInline, CollectionIdentifierInline]

    def forms_valid(self, form, inlines):
        messages.info(self.request, ugettext_lazy("You have successfully added your collection."))
        obj = form.save()
        obj.set_revision(self.request.user)
        return super(CollectionAddView, self).forms_valid(form, inlines)

    def get_success_url(self):
        return reverse_lazy('telemeta-collection-detail', kwargs={'public_id':self.object.code})

    @method_decorator(permission_required('telemeta.add_mediacollection'))
    def dispatch(self, *args, **kwargs):
        return super(CollectionAddView, self).dispatch(*args, **kwargs)


class CollectionCopyView(CollectionAddView):

    template_name = 'telemeta/collection_edit.html'

    def get_initial(self):
        return model_to_dict(self.get_object())

    def get_context_data(self, **kwargs):
        context = super(CollectionCopyView, self).get_context_data(**kwargs)
        collection = self.get_object()
        context['collection'] = collection
        return context

    @method_decorator(permission_required('telemeta.add_mediacollection'))
    def dispatch(self, *args, **kwargs):
        return super(CollectionCopyView, self).dispatch(*args, **kwargs)



class CollectionEpubView(BaseEpubMixin, View):
    "Download collection data embedded in an EPUB3 file"

    model = MediaCollection

    def get_object(self):
        return MediaCollection.objects.get(public_id=self.kwargs['public_id'])

    def get(self, request, *args, **kwargs):
        collection = self.get_object()
        corpus = collection.corpus.all()[0]
        self.setup_epub(corpus, collection=collection)
        if not os.path.exists(self.path):
            self.write_book()
        epub_file = open(self.path, 'rb')
        response = HttpResponse(epub_file.read(), content_type='application/epub+zip')
        response['Content-Disposition'] = "attachment; filename=%s" % self.filename + '.epub'
        return response

    # @method_decorator(login_required)
    # @method_decorator(permission_required('telemeta.can_download_collection_epub'))
    def dispatch(self, *args, **kwargs):
        return super(CollectionEpubView, self).dispatch(*args, **kwargs)




#!/usr/bin/python

import os, sys, string
import logging


class Logger:
    """A logging object"""

    def __init__(self, file):
        self.logger = logging.getLogger('myapp')
        self.hdlr = logging.FileHandler(file)
        self.formatter = logging.Formatter('%(asctime)s %(message)s')
        self.hdlr.setFormatter(self.formatter)
        self.logger.addHandler(self.hdlr)
        self.logger.setLevel(logging.INFO)


class TelemetaTranscode(object):
    """docstring for TelemetaTranscode"""

    threads = 4
    source_formats = ['webm', 'mp4']
    dest_formats = {
                   'mp3' : '-vn -acodec libmp3lame -aq 6',
                   'ogg' : '-vn -acodec libvorbis -aq 6',
                   'mp4' : '-vcodec libx264 -threads ' + str(threads) + \
                           ' -c:v libx264 -crf 17 -maxrate 1100k -bufsize 1835k -acodec libfaac -ab 96k',
                   'png' : '',
                   'webm' : '-vcodec libvpx -threads ' + str(threads) + \
                           ' -c:v libvpx -crf 17 -b:v 1100k',
                  }


    def __init__(self, args):
        self.args = args
        self.log_file = args[-1]
        self.root_dir = args[-2]
        self.logger = Logger(self.log_file)


    def get_ext_in_dir(self, extension, root):
        files = os.listdir(root)
        exts = []
        for f in files:
            name, ext = os.path.splitext(f)
            ext = ext[1:]
            if not ext in exts:
                exts.append(ext)
        return extension in exts

    def run(self):
        for root, dirs, files in os.walk(self.root_dir):
            for file in files:
                path = os.path.abspath(root + os.sep + file)
                name, ext = os.path.splitext(file)
                ext = ext[1:]
                if ext in self.source_formats:
                    for format, ffmpeg_args in self.dest_formats.iteritems():
                        local_file = name + '.' + format
                        dest = os.path.abspath(root + os.sep + local_file)
                        local_files = os.listdir(root)
                        if not (local_file in local_files or self.get_ext_in_dir(format, root)) or '--force' in self.args:
                            if ext == 'webm' and format == 'ogg':
                                ffmpeg_args = '-vn -acodec copy'
                            command = 'ffmpeg -loglevel 0 -i "' + path + '" ' + ffmpeg_args + ' -y "' + dest + '"'
                            self.logger.logger.info(command)
                            if not '--dry-run' in self.args:
                                os.system(command)
                            else:
                                print command


if __name__ == '__main__':
    t = TelemetaTranscode(sys.argv[1:])
    t.run()



# -*- coding: utf-8 -*-
from telemeta.views.core import *
from telemeta.models import *
import uuid

class NewPlaylistView(object):

    def display(self, request, type=None):

        self.type=type

        template_name = loader.get_template('search/addplaylist.html')

        idlist = request.POST.getlist('selected_items_list')
        itemlist = []
        if self.type =="item":
            for itemid in idlist:
                itemlist.append(MediaItem.objects.all().get(id=itemid))

        if self.type =="collection":
            for itemid in idlist:
                itemlist.append(MediaCollection.objects.all().get(id=itemid))

        context = RequestContext(request, {
                'selected_items_list': itemlist,
                'existing_playlists': Playlist.objects.all().filter(author=request.user),
                'type':self.type,
                'lastquerypath':request.POST.get('lastquerypath')})
        return HttpResponse(template_name.render(context))

    def addToPlaylist(self, request, type=None):
        self.type=type

        template_name = loader.get_template('search/confirmation_add_playslist.html')

        idlist = request.POST.getlist('item_id')
        selected_playlist_id = request.POST.get('playlist_id')
        selected_playlist = Playlist.objects.all().get(id=selected_playlist_id)

        itemlist = []
        if self.type == "item":
            for itemid in idlist:
                itemlist.append(MediaItem.objects.all().get(id=itemid))

            for item in itemlist:
                new_id = uuid.uuid4()
                PlaylistResource.objects.get_or_create(resource_type='item',resource_id=item.id,playlist=selected_playlist,defaults={'public_id':new_id})
        else:
            for itemid in idlist:
                itemlist.append(MediaCollection.objects.all().get(id=itemid))

            for item in itemlist:
                new_id = uuid.uuid4()
                PlaylistResource.objects.get_or_create(resource_type='collection',resource_id=item.id,playlist=selected_playlist,defaults={'public_id':new_id})

        context = RequestContext(request, {
                'selected_items_list': itemlist,
                'type':self.type,
                'lastquerypath': request.POST.get('lastquerypath')})
        return HttpResponse(template_name.render(context))





# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *
from saved_searches.models import SavedSearch


class HomeView(object):
    """Provide general web UI methods"""

    def home(self, request):
        """Render the index page"""

        template = loader.get_template('telemeta/home.html')

        N = 3   # max number of pub items

        sound_items = MediaItem.objects.sound_public()
        count = sound_items.count()

        if count == 0:
            sound_pub_item = None
            sound_pub_items = None

        elif count == 1:
            sound_pub_item = sound_items[0]
            sound_pub_items = [sound_items[0], sound_items[0]]

        elif count == 2:
            sound_pub_item = sound_items[0]
            sound_pub_items = [sound_items[0], sound_items[1]]

        elif count > 2:
            indexes = random.sample(range(count-1), N)
            sound_pub_item = sound_items[indexes[0]]
            sound_pub_items = [sound_items[indexes[i]] for i in range(1, N)]

        revisions = get_revisions(25)
        context = RequestContext(request, {
                    'page_content': pages.get_page_content(request, 'home', ignore_slash_issue=True),
                    'revisions': revisions,  'sound_pub_items': sound_pub_items,
                    'sound_pub_item': sound_pub_item })
        return HttpResponse(template.render(context))

    def lists(self, request):
        """Render the home page"""

        if request.user.is_authenticated():
            template='telemeta/lists.html'
            playlists = get_playlists(request)
            revisions = get_revisions(100)
            user_revisions = get_revisions(25, request.user)
            return render(request, template, {'playlists': playlists,
                                              'revisions': revisions, 'user_revisions': user_revisions })
        else:
            template = 'telemeta/messages.html'
            mess = ugettext('Access not allowed')
            title = ugettext('Lists') + ' : ' + mess
            description = ugettext('Please login or contact the website administator to get a private access.')
            messages.error(request, title)
            return render(request, template, {'description' : description})

    def handle_oai_request(self, request):
        host = request.META['HTTP_HOST']
        datasource  = TelemetaOAIDataSource()
        repository_name = settings.TELEMETA_ORGANIZATION
        url         = 'http://' + host + request.path
        admin       = settings.ADMINS[0][1]
        provider    = oai.DataProvider(datasource, repository_name, url, admin)
        args        = request.GET.copy()
        args.update(request.POST)
        return HttpResponse(provider.handle(args), mimetype='text/xml')

    def render_flatpage(self, request, path):
        try:
            content = pages.get_page_content(request, path)
        except pages.MalformedPagePath:
            return redirect(request.path + '/')

        if isinstance(content, pages.PageAttachment):
            return HttpResponse(content, content.mimetype())
        else:
            return render(request, 'telemeta/flatpage.html', {'page_content': content })

    def logout(self, request):
        auth.logout(request)
        return redirect('telemeta-home')

    def complete_location(self, request, with_items=True):
        input = request.REQUEST

        token = input['q']
        limit = int(input['limit'])
        if with_items:
            locations = MediaItem.objects.all().locations()
        else:
            locations = Location.objects.all()

        locations = locations.filter(name__istartswith=token).order_by('name')[:limit]
        data = [unicode(l) + " (%d items)" % l.items().count() for l in locations]

        return HttpResponse("\n".join(data))

    @method_decorator(login_required)
    def users(self, request):
        users = User.objects.all().order_by('last_name')
        return render(request, 'telemeta/users.html', {'users': users})



# -*- coding: utf-8 -*-
# Copyright (C) 2007 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>

from telemeta.models.core import *
from telemeta.util.unaccent import unaccent
import re
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from telemeta.models.query import *
from django.forms import ModelForm


class Location(ModelCore):
    "Locations"
    OTHER_TYPE  = 0
    CONTINENT   = 1
    COUNTRY     = 2
    TYPE_CHOICES     = ((COUNTRY, _('country')), (CONTINENT, _('continent')), (OTHER_TYPE, _('other')))

    name             = CharField(_('name'), unique=True, max_length=150, required=True)
    type             = IntegerField(_('type'), choices=TYPE_CHOICES, default=OTHER_TYPE, db_index=True)
    complete_type    = ForeignKey('LocationType', related_name="locations", verbose_name=_('complete type'))
    current_location = WeakForeignKey('self', related_name="past_names",
                                      verbose_name=_('current location'))
    latitude         = FloatField(null=True)
    longitude        = FloatField(null=True)
    is_authoritative = BooleanField(_('authoritative'))

    objects = LocationManager()

    def items(self):
        from telemeta.models import MediaItem
        return MediaItem.objects.by_location(self)

    def collections(self):
        from telemeta.models import MediaCollection
        return MediaCollection.objects.by_location(self)

    def ancestors(self, direct=False):
        q = Q(descendant_relations__location=self)
        if direct:
            q &= Q(descendant_relations__is_direct=True)
        return Location.objects.filter(q)

    def descendants(self, direct=False):
        q = Q(ancestor_relations__ancestor_location=self)
        if direct:
            q &= Q(ancestor_relations__is_direct=True)
        return Location.objects.filter(q)

    def apparented(self):
        return Location.objects.filter(
                Q(pk=self.id) |
                Q(ancestor_relations__ancestor_location=self) |
                Q(current_location=self.id)).distinct()

    def add_child(self, other):
        LocationRelation.objects.create(location=other, ancestor_location=self, is_direct=True)
        for location in self.ancestors():
            #FIXME: might raise Duplicate Entry
            LocationRelation.objects.create(location=other, ancestor_location=location)

    def add_parent(self, other):
        LocationRelation.objects.create(location=self, ancestor_location=other, is_direct=True)
        for location in self.descendants():
            #FIXME: might raise Duplicate Entry
            LocationRelation.objects.create(location=location, ancestor_location=other)

    def countries(self):
        if self.type == self.COUNTRY:
            return Location.objects.filter(pk=self.id)
        return self.ancestors().filter(type=self.COUNTRY)

    def continents(self):
        if self.type == self.CONTINENT:
            return Location.objects.filter(pk=self.id)
        return self.ancestors().filter(type=self.CONTINENT)

    class Meta(MetaCore):
        db_table = 'locations'
        verbose_name = _('location')
        verbose_name_plural = _('locations')
        ordering = ['name']

    def __unicode__(self):
        return self.name

    def flatname(self):
        if self.type != self.COUNTRY and self.type != self.CONTINENT:
            raise Exception("Flat names are only supported for countries and continents")

        map = Location.objects.flatname_map()
        for flatname in map:
            if self.id == map[flatname]:
                return flatname

        return None

    def paths(self):
        #FIXME: need to handle multiple (polyhierarchical) paths
        path = []
        location = self
        while location:
            path.append(location)
            try:
                location = location.ancestors(direct=True)[0]
            except IndexError:
                location = None
        return [path]

    def fullnames(self):
        names = []
        for path in self.paths():
            names.append(u', '.join([unicode(l) for l in path]))
        return names

    def listnames(self):
        names = []
        for path in self.paths():
            for l in path:
                names.append(unicode(l))
        return names


class LocationType(ModelCore):
    "Location types"
    code = CharField(_('identifier'), max_length=64, unique=True, required=True)
    name = CharField(_('name'), max_length=150, required=True)

    def __unicode__(self):
        return self.name

    class Meta(MetaCore):
        db_table = 'location_types'
        ordering = ['name']
        verbose_name_plural = _('location types')

class LocationAlias(ModelCore):
    "Location aliases"
    location         = ForeignKey('Location', related_name="aliases", verbose_name=_('location'))
    alias            = CharField(_('alias'), max_length=150, required=True)
    is_authoritative = BooleanField(_('authoritative'))

    def __unicode__(self):
        return self.alias

    class Meta(MetaCore):
        db_table = 'location_aliases'
        unique_together = (('location', 'alias'),)
        verbose_name_plural = _('location aliases')
        ordering = ['alias']

class LocationRelation(ModelCore):
    "Location relations"
    location             = ForeignKey('Location', related_name="ancestor_relations", verbose_name=_('location'))
    ancestor_location    = ForeignKey('Location', related_name="descendant_relations",  verbose_name=_('ancestor location'))
    is_direct            = BooleanField(db_index=True)
    is_authoritative = BooleanField(_('authoritative'))

    class Meta(MetaCore):
        db_table = 'location_relations'
        unique_together = ('location', 'ancestor_location')
        ordering = ['ancestor_location__name']
        verbose_name_plural = _('location relations')

    def __unicode__(self):
        sep = ' > '
        if not self.is_direct:
            sep = ' >..> '
        return unicode(self.ancestor_location) + sep + unicode(self.location)


class LocationForm(ModelForm):
    class Meta:
        model = Location

    def __init__(self, *args, **kwds):
        super(LocationForm, self).__init__(*args, **kwds)
#        self.fields['name'].queryset = Location.objects.order_by('name')




from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.core.files.base import ContentFile
from telemeta.models import *
from telemeta.util.unaccent import unaccent
from telemeta.cache import TelemetaCache
import urllib

try:
    from django.utils.text import slugify
except ImportError:
    def slugify(string):
        killed_chars = re.sub('[\(\),]', '', string)
        return re.sub(' ', '_', killed_chars)

def beautify(string):
    return os.path.splitext(string)[0].replace('_',' ')

class Command(BaseCommand):
    args = "<media_file1 [media_file2 ...]>"
    help = "Download and import a media item"
    option_list = BaseCommand.option_list + (
            make_option('--collection-code',
                action='store',
                dest='code',
                default='default',
                metavar = '<code>',
                help='collection code'),
            make_option('--collection-title',
                action='store',
                dest='title',
                default='default',
                metavar = '<title>',
                help='collection title'),
            )

    cache_data = TelemetaCache(settings.TELEMETA_DATA_CACHE_DIR)
    cache_export = TelemetaCache(settings.TELEMETA_EXPORT_CACHE_DIR)

    urls = []

    def handle(self, *args, **options):
        if len(args) < 1:
            return
        if options['title']:
            self.title = options['title']
        if options['code']:
            self.code = options['code']
        for file in args:
            self.urls.append('file://' + file)

        collections = MediaCollection.objects.filter(code=self.code)
        if not collections:
            # create a new collection
            collection = MediaCollection(code=self.code, title=self.title)
            collection.public_access = 'full'
            collection.save()
        else:
            collection = collections[0]

        for url in self.urls:
            basename = os.path.basename(url)
            code = slugify(basename)
            title = beautify(basename)
            items = MediaItem.objects.filter(code=code)
            if not items:
                item = MediaItem(collection=collection, code=code, title=title)
                item.save()
            else:
                print 'cleaning up', code
                item = items[0]
                self.cache_data.delete_item_data(code)
                self.cache_export.delete_item_data(code)
                flags = MediaItemTranscodingFlag.objects.filter(item=item)
                analyses = MediaItemAnalysis.objects.filter(item=item)
                for flag in flags:
                    flag.delete()
                for analysis in analyses:
                    analysis.delete()

            print 'fetching: ' + url
            file = urllib.urlopen(url)
            file_content = ContentFile(file.read())
            item.title = title
            item.file.save(code, file_content)
            item.public_access = 'full'
            item.save()
            print 'item created: ', collection, code

        print 'done importing', len(self.urls), 'items'



from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from telemeta.models import *
import datetime, time, calendar, itertools
import numpy as np

import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from matplotlib.backends.backend_pdf import PdfPages


SECOND = 1
MINUTE = SECOND * 60
HOUR = MINUTE * 60
DAY = HOUR * 24
MONTH = DAY * 30


class Command(BaseCommand):
    help = "export MediaItem revisions vs. dates to a matplotlib PDF file"
    """ info :
    http://www.geophysique.be/2012/06/14/matplotlib-datetimes-tutorial-03-grouping-sparse-data/
    http://matplotlib.org/examples/pylab_examples/date_demo2.html
    http://matplotlib.org/examples/api/date_demo.html
    """
    args = 'year month day'
    binning = 7*DAY

    option_list = BaseCommand.option_list + (
          make_option('-y', '--year',
            dest='year',
            help='year of the first revision'),
          make_option('-m', '--month',
            dest='month',
            help='month of the first revision'),
          make_option('-d', '--day',
            dest='day',
            help='day of the first revision'),
    )
    def group(self, di):
        return int(calendar.timegm(di.timetuple()))/self.binning

    def handle(self, *args, **kwargs):
        limit_date = datetime.datetime(int(kwargs.get('year')), int(kwargs.get('month')), int(kwargs.get('day')))
        years    = mdates.YearLocator()
        months    = mdates.MonthLocator(range(1,13), bymonthday=1, interval=3)
        mondays   = mdates.WeekdayLocator(mdates.MONDAY)
        monthsFmt = mdates.DateFormatter("%b '%y")
        yearsFmt = mdates.DateFormatter('%Y')

        revisions = Revision.objects.filter(time__gte=limit_date)
        list_of_dates = [r.time for r in revisions]
        grouped_dates = [[datetime.datetime(*time.gmtime(d*self.binning)[:6]), len(list(g))] \
                            for d,g in itertools.groupby(list_of_dates, self.group)]
        grouped_dates = zip(*grouped_dates)
        
        revs = np.array(grouped_dates[1])
        revs_mean = np.mean(revs)

        fig = plt.figure()
        ax = fig.add_subplot(111, ylabel='Revisions by week (mean='+str(np.round(revs_mean, 1))+')')
        ax.plot_date(grouped_dates[0], grouped_dates[1] , '-')
        ax.xaxis.set_major_locator(months)
        ax.xaxis.set_major_formatter(monthsFmt)
        ax.xaxis.set_minor_locator(mondays)
        ax.autoscale_view()
        ax.grid(True)
        fig.autofmt_xdate()

        plt.savefig('/tmp/telemeta-revisions.png')
        plt.savefig('/tmp/telemeta-revisions.pdf')

        #plt.show()




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# -*- coding: utf-8 -*-
#
# Copyright (C) 2007 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Olivier Guilyardi <olivier@samalyse.com>

import os
import sys
import time
from django.core.management import setup_environ

def print_usage(toolname):
    print "Usage: " + toolname + " <project_dir> <backup_dir>"
    print "  project_dir: the directory of the Django project which hosts Telemeta"
    print "  backup_dir: the destination backup folder (must exist)"

def write_readme(dest_dir, coll_num):
    readme = open(dest_dir + "/" + "README", "w")
    timestr = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime())
    readme.write("Telemeta Backup\n\n")
    readme.write("- date: " + timestr + "\n")
    readme.write("- number of collections: " + str(coll_num) + "\n\n")
    readme.close()

def backup(dest_dir):
    from telemeta.models import MediaCollection
    from telemeta.backup import CollectionSerializer

    collections = MediaCollection.objects.order_by('id')
    count = collections.count()

    print "Writing README file..",
    write_readme(dest_dir, count)
    print "Done."

    i = 0
    for collection in collections:
        if i % 100 == 0:
            set_dir = dest_dir + ("/collections-%d-%d" % (i+1, i+100))
            os.mkdir(set_dir)
        i += 1
        print "Processing collection %d/%d (%d%%) with id: %s.. " \
            % (i, count, i*100/count, collection.id),
        sys.stdout.flush()
        serializer = CollectionSerializer(collection)
        serializer.backup(set_dir)
        print "Done"

def run():
    if len(sys.argv) != 3:
        print_usage(os.path.basename(sys.argv[0]))
        sys.exit(1)
    else:
        project_dir = sys.argv[1]
        backup_dir = sys.argv[2]
        sys.path.append(project_dir)
        import settings
        setup_environ(settings)
        backup(backup_dir)

if __name__ == '__main__':
    run()



# -*- coding: utf-8 -*-
# Copyright (C) 2007-2010 Samalyse SARL
# Copyright (C) 2010-2011 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          David LIPSZYC <davidlipszyc@gmail.com>
#          Guillaume Pellerin <yomguy@parisson.com>

from telemeta.models.core import *
from django.utils.translation import ugettext_lazy as _


class Instrument(ModelCore):
    "Instrument used in the item"
    name = CharField(_('name'), required=True)
    notes = TextField(_('notes'))

    class Meta(MetaCore):
        db_table = 'instruments'
        ordering = ['name']

    def __unicode__(self):
        return self.name

class InstrumentAlias(ModelCore):
    "Instrument other name"
    name = CharField(_('name'), required=True)
    notes = TextField(_('notes'))

    class Meta(MetaCore):
        db_table = 'instrument_aliases'
        verbose_name_plural = _('instrument aliases')
        ordering = ['name']

    def __unicode__(self):
        return self.name

class InstrumentRelation(ModelCore):
    "Instrument family"
    instrument = ForeignKey('Instrument', related_name="parent_relation",
                                   verbose_name=_('instrument'))
    parent_instrument = ForeignKey('Instrument', related_name="child_relation",
                                   verbose_name=_('parent instrument'))

    class Meta(MetaCore):
        db_table = 'instrument_relations'
        unique_together = (('instrument', 'parent_instrument'),)

    def __unicode__(self):
        sep = ' > '
        return self.parent_instrument.name + sep + self.instrument.name

class InstrumentAliasRelation(ModelCore):
    "Instrument family other name"
    alias = ForeignKey('InstrumentAlias', related_name="other_name",
                            verbose_name=_('alias'))
    instrument = ForeignKey('Instrument', related_name="relation",
                            verbose_name=_('instrument'))

    def __unicode__(self):
        sep = ' : '
        return self.alias.name + sep + self.instrument.name

    class Meta(MetaCore):
        db_table = 'instrument_alias_relations'
        unique_together = (('alias', 'instrument'),)




import os

DATABASE_ENGINE = 'sqlite3'
DATABASE_NAME = 'notes.db'

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'haystack',
    'saved_searches',
    'notes',
]

ROOT_URLCONF = 'test_urls'

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.core.context_processors.auth",
    "django.core.context_processors.request",
)

HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.whoosh_backend.WhooshEngine',
        'PATH': os.path.join(os.path.dirname(__file__), 'whoosh_index'),
    }
}
# Set this low to simulate multiple pages.
HAYSTACK_SEARCH_RESULTS_PER_PAGE = 1



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2013 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Guillaume Pellerin <yomguy@parisson.com>


import time
from telemeta.util.xmltodict2 import *


class KDEnLiveSession(object):

    def __init__(self, path):
        self.session = xmltodict(path)

    def entries(self):
        entries = []
        for attr in self.session['children']:
            if 'playlist' in attr['name'] and 'children' in attr:
                for att in attr['children']:
                    if 'entry' in att['name'] and att['attributes']['producer'] != 'black':
                        entries.append(att['attributes'])
        return entries

    def video_entries(self):
        entries = []
        for attr in self.session['children']:
            if 'playlist' in attr['name'] and 'children' in attr:
                for att in attr['children']:
                    if 'entry' in att['name'] and att['attributes']['producer'] != 'black' \
                            and not 'audio' in att['attributes']['producer']:
                        entries.append(att['attributes'])
        return entries

    def entries_sorted(self):
        return sorted(self.entries(), key=lambda k: int(k['in']), reverse=False)

    def entries_video_seconds(self):
        fps = float(self.profile()['frame_rate_num'])
        list = []
        entries = self.video_entries()
        for i in range(0,len(entries)):
            id = entries[i]['producer'].split('_')[0]
            t_in = int(entries[i]['in'])/fps
            t_out = int(entries[i]['out'])/fps

            if i == 0:
                t = 0
            else:
                t = list[i-1]['t'] + int(entries[i-1]['out'])/fps - int(entries[i-1]['in'])/fps

            list.append({'id' : id, 't': t, 'in': t_in , 'out': t_out })

        return list

    def cuts(self, entries):
        i = 0
        cuts = [0, ]
        for entry in entries:
            if i > 0:
                cuts.append(cuts[i-1] + int(entries[i]['in'])-int(entries[i-1]['out']))
            i += 1
        return cuts

    def first_video_frame(self):
        return int(self.entries_sorted()[0]['in'])

    def profile(self):
        for attr in self.session['children']:
            if 'profile' in attr['name']:
                return attr['attributes']

    def fix_text(self, text):
        try:
            s = text.split(' ')
            i = int(s[1])
            s.insert(2, ':')
            return ' '.join(s)
        except:
            return text

    def markers(self, offset=0, from_first_marker=False):
        """ by default return a dict of markers with timecodes relative to an origin

            if from_first_marker=False: the origin is the first entry timecode
            if from_first_marker=True: the origin is the first entry timecode before the first marker

            offset: general origin offset
        """

        abs_time = 0
        markers = []
        i = 0
        entries = self.entries_video_seconds()

        for attr in self.session['children']:
            if 'kdenlivedoc' in attr['name']:

                for att in attr['children']:
                    if 'markers' in att['name'] and 'children' in att.keys():

                        for at in att['children']:
                            if 'marker' in at['name']:

                                marker_time = float(at['attributes']['time'].replace(',','.'))
                                id = at['attributes']['id']
                                rel_time = 0

                                for entry in entries:
                                    if marker_time >= entry['in'] and marker_time <= entry['out'] and id == entry['id']:
                                        if i == 0 and from_first_marker:
                                            abs_time = entry['t']
                                        rel_time = entry['t'] + (marker_time - entry['in']) - abs_time + offset
                                        break

                                at['attributes']['time'] = rel_time
                                at['attributes']['session_timecode'] = time.strftime('%H:%M:%S', time.gmtime(rel_time))
                                at['attributes']['comment'] = self.fix_text(at['attributes']['comment'])
                                markers.append(at['attributes'])

                            i += 1
        return markers




#!/usr/bin/python

import os, sys, psutil
import datetime
from ebml.utils.ebml_data import *

class FixCheckMedia(object):

    def __init__(self, dir, tmp_dir):
        self.dir = dir
        self.tmp_dir = tmp_dir
        if not os.path.exists(self.tmp_dir):
            os.makedirs(self.tmp_dir)

    def process(self):
        webm_fixed_log = 'webm.fixed'
        webm_tofix_log = 'webm.tofix'
        mp3_fixed_log = 'mp3.fixed'
        mp3_tofix_log = 'mp3.tofix'

        for root, dirs, files in os.walk(self.dir):
            for filename in files:
                source = root + os.sep + filename
                name = os.path.splitext(filename)[0]
                ext = os.path.splitext(filename)[1][1:]

                if ext == 'webm' and os.path.getsize(source):
                    dir_files = os.listdir(root)

                    if not webm_fixed_log in dir_files:
                        print source
                        self.fix_webm(source)
                        f = open(root + os.sep + webm_fixed_log, 'w')
                        f.close()
                        if os.path.exists(root + os.sep + webm_tofix_log):
                            os.remove(root + os.sep + webm_tofix_log)

                    if mp3_tofix_log in dir_files or not mp3_fixed_log in dir_files:
                        for file in dir_files:
                            dest_ext = os.path.splitext(file)[1][1:]
                            if dest_ext == 'mp3':
                                dest = root + os.sep + file
                                print dest
                                self.fix_mp3(source, dest)
                                f = open(root + os.sep + mp3_fixed_log, 'w')
                                f.close()
                                if os.path.exists(root + os.sep + mp3_tofix_log):
                                    os.remove(root + os.sep + mp3_tofix_log)
                                #break


    def hard_fix_webm(self, path):
        try:
            tmp_file = self.tmp_dir + 'out.webm '
            command = 'ffmpeg -loglevel 0 -i "'+ path + '" -vcodec libvpx -vb 500k -acodec libvorbis -aq 7 -f webm -y "' + tmp_file + '" > /dev/null'
            print command
            os.system(command)
            command = 'mv '  + tmp_file + path
            os.system(command)
        except:
            pass


    def fix_webm(self, path):
        try:
            tmp_file = self.tmp_dir + 'out.webm'
            command = '/usr/local/bin/ffmpeg -loglevel 0 -i "' + path + '" -vcodec copy -acodec copy -f webm -y "' + tmp_file + '" > /dev/null'
            print command
            os.system(command)
            ebml_obj = EBMLData(tmp_file)
            offset = ebml_obj.get_first_cluster_seconds()
            command = '/usr/local/bin/ffmpeg -loglevel 0 -ss ' + str(offset) + ' -i "' + tmp_file + '" -vcodec copy -acodec copy -f webm -y "' + path + '" > /dev/null'
            print command
            os.system(command)
        except:
            pass

    def fix_mp3(self, source, path):
        try:
            command = '/usr/local/bin/ffmpeg -loglevel 0 -i "'+ source + '" -vn -aq 6 -y "' + path + '" > /dev/null'
            print command
            os.system(command)
        except:
            pass

def get_pids(name, args=None):
    """Get a process pid filtered by arguments and uid"""
    pids = []
    for proc in psutil.process_iter():
        if proc.cmdline:
            if name == proc.name:
                if args:
                    if args in proc.cmdline:
                        pids.append(proc.pid)
                else:
                    pids.append(proc.pid)
    return pids

dir = sys.argv[-2]
tmp_dir = sys.argv[-1]

path =  os.path.abspath(__file__)
pids = get_pids('python2.6',args=path)

print datetime.datetime.now()
if len(pids) <= 1:
    print 'starting process...'
    f = FixCheckMedia(dir, tmp_dir)
    f.process()
    print 'process finished.\n'
else:
    print 'already started !\n'




# -*- coding: utf-8 -*-

# Copyright (C) 2010-2015 Parisson SARL
# Copyright (C) 2007-2010 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Authors: Olivier Guilyardi <olivier@samalyse.com>
#          Guillaume Pellerin <yomguy@parisson.com>


from telemeta.views.core import *
from telemeta.views.marker import *
import timeside.core


class ItemBaseMixin(TelemetaBaseMixin):

    graphers = timeside.core.processor.processors(timeside.core.api.IGrapher)
    decoders = timeside.core.processor.processors(timeside.core.api.IDecoder)
    encoders = timeside.core.processor.processors(timeside.core.api.IEncoder)
    analyzers = timeside.core.processor.processors(timeside.core.api.IAnalyzer)
    value_analyzers = timeside.core.processor.processors(timeside.core.api.IValueAnalyzer)

    export_enabled = getattr(settings, 'TELEMETA_DOWNLOAD_ENABLED', True)
    export_formats = getattr(settings, 'TELEMETA_DOWNLOAD_FORMATS', ('mp3', 'wav'))
    default_grapher_id = getattr(settings, 'TIMESIDE_DEFAULT_GRAPHER_ID', ('waveform_simple'))
    default_grapher_sizes = getattr(settings, 'TIMESIDE_DEFAULT_GRAPHER_SIZES', ['346x130', ])
    auto_zoom = getattr(settings, 'TIMESIDE_AUTO_ZOOM', False)

    def get_export_formats(self):
        formats = []
        for encoder in self.encoders:
            if encoder.file_extension() in self.export_formats:
                formats.append({'name': encoder.format(),
                                    'extension': encoder.file_extension()})
        return formats

    def get_graphers(self):
        graphers = []
        for grapher in self.graphers:
            if grapher.id() == self.default_grapher_id:
                graphers.insert(0, {'name':grapher.name(), 'id': grapher.id()})
            elif not hasattr(grapher, '_staging'):
                graphers.append({'name':grapher.name(), 'id': grapher.id()})
            elif not grapher._staging:
                graphers.append({'name':grapher.name(), 'id': grapher.id()})
        return graphers

    def get_grapher(self, id):
        for grapher in self.graphers:
            if grapher.id() == id:
                break
        return grapher

    def get_export_formats(self):
        formats = []
        for encoder in self.encoders:
            if encoder.file_extension() in self.export_formats:
                formats.append({'name': encoder.format(),
                                    'extension': encoder.file_extension()})
        return formats

    def item_previous_next(self, item):
        """Get previous and next items inside the collection of the item"""

        pks = []
        items = MediaItem.objects.filter(collection=item.collection)
        items = items.order_by('code', 'old_code')

        if len(items) > 1:
            for it in items:
                pks.append(it.pk)
            for pk in pks:
                if pk == item.pk:
                    if pk == pks[0]:
                        previous_pk = pks[-1]
                        next_pk = pks[1]
                    elif pk == pks[-1]:
                        previous_pk = pks[-2]
                        next_pk = pks[0]
                    else:
                        previous_pk = pks[pks.index(pk)-1]
                        next_pk = pks[pks.index(pk)+1]
                    for it in items:
                        if it.pk == previous_pk:
                            previous = it
                        if it.pk == next_pk:
                            next = it
                    previous = previous.public_id
                    next = next.public_id
        else:
             previous = item.public_id
             next = item.public_id
        return previous, next


class ItemView(ItemBaseMixin):
    """Provide Item web UI methods"""

    def item_detail(self, request, public_id=None, marker_id=None, width=None, height=None,
                        template='telemeta/mediaitem_detail.html'):
        """Show the details of a given item"""

        # get item with one of its given marker_id
        if not public_id and marker_id:
            marker = get_object_or_404(MediaItemMarker, public_id=marker_id)
            item_id = marker.item_id
            item = MediaItem.objects.get(id=item_id)
        else:
            item = get_object_or_404(MediaItem, public_id=public_id)

        access = get_item_access(item, request.user)

        if access == 'none':
            mess = ugettext('Access not allowed')
            title = ugettext('Item') + ' : ' + public_id + ' : ' + mess
            description = ugettext('Please login or contact the website administator to get a private access.')
            messages.error(request, title)
            return render(request, 'telemeta/messages.html', {'description' : description})

        previous, next = self.item_previous_next(item)

        mime_type = item.mime_type
        if mime_type and mime_type != 'none' :
            if 'quicktime' in mime_type:
                mime_type = 'video/mp4'

        playlists = get_playlists_names(request)
        related_media = MediaItemRelated.objects.filter(item=item)
        check_related_media(related_media)
        revisions = Revision.objects.filter(element_type='item', element_id=item.id).order_by('-time')
        if revisions:
            last_revision = revisions[0]
        else:
            last_revision = None

        format = ''
        if Format.objects.filter(item=item):
            format = item.format.get()

        return render(request, template,
                    {'item': item, 'export_formats': self.get_export_formats(),
                    'visualizers': self.get_graphers(), 'auto_zoom': self.auto_zoom,
                    'audio_export_enabled': self.export_enabled,
                    'previous' : previous, 'next' : next, 'marker': marker_id, 'playlists' : playlists,
                    'access': access, 'width': width, 'height': height,
                    'related_media': related_media, 'mime_type': mime_type, 'last_revision': last_revision,
                    'format': format,
                    })

    def related_media_item_stream(self, request, item_public_id, media_id):
        item = get_object_or_404(MediaItem, code=item_public_id)
        media = get_object_or_404(MediaItemRelated, item=item, id=media_id)
        if media.file:
            response = StreamingHttpResponse(stream_from_file(media.file.path), content_type=media.mime_type)
        else:
            raise Http404
        return response

    def related_media_item_download(self, request, item_public_id, media_id):
        item = get_object_or_404(MediaItem, code=item_public_id)
        media = get_object_or_404(MediaItemRelated, item=item, id=media_id)
        if media.file:
            filename = media.file.path.split(os.sep)[-1]
            response = StreamingHttpResponse(stream_from_file(media.file.path), content_type=media.mime_type)
            response['Content-Disposition'] = 'attachment; ' + 'filename=' + filename
        else:
            raise Http404
        return response

    @method_decorator(permission_required('telemeta.change_mediaitem'))
    def related_media_edit(self, request, public_id, template):
        item = get_object_or_404(MediaItem, code=public_id)
        MediaItemRelatedFormSet = inlineformset_factory(MediaItem, MediaItemRelated, form=MediaItemRelatedForm)
        if request.method == 'POST':
            formset = MediaItemRelatedFormSet(data=request.POST, files=request.FILES, instance=item)
            if formset.is_valid():
                formset.save()
                item.set_revision(request.user)
                return redirect('telemeta-item-edit', public_id)
        else:
            formset = MediaItemRelatedFormSet(instance=item)
        return render(request, template, {'item': item, 'formset': formset,})


    @method_decorator(permission_required('telemeta.delete_mediaitem'))
    def item_delete(self, request, public_id):
        """Delete a given item"""
        item = MediaItem.objects.get(public_id=public_id)
        revisions = Revision.objects.filter(element_type='item', element_id=item.id)
        for revision in revisions:
            revision.delete()
        collection = item.collection
        item.delete()
        return redirect('telemeta-collection-detail', collection.code)

    def item_analyze_xml(self, request, public_id):
        item = MediaItem.objects.get(public_id=public_id)
        analyses = item.analysis.all()
        analyzers = []
        for analysis in analyses:
            analyzers.append(analysis.to_dict())
        mime_type = 'text/xml'
        response = HttpResponse(self.cache_data.get_analyzer_xml(analyzers), content_type=mime_type)
        response['Content-Disposition'] = 'attachment; filename='+public_id+'.xml'
        return response

    def item_visualize(self, request, public_id, grapher_id, width, height):
        try:
            width = int(width)
            height = int(height)
        except:
            pass

        if not isinstance(width, int) or not isinstance(height, int):
            size = self.default_grapher_sizes[0]
            width = int(size.split('x')[0])
            height = int(size.split('x')[1])

        item = MediaItem.objects.get(public_id=public_id)
        mime_type = 'image/png'
        grapher = self.get_grapher(grapher_id)

        if grapher.id() != grapher_id:
            raise Http404

        size = str(width) + '_' + str(height)
        image_file = '.'.join([public_id, grapher_id, size, 'png'])

        # FIX waveform grapher name change
        old_image_file = '.'.join([public_id, 'waveform', size, 'png'])
        if 'waveform_centroid' in grapher_id and self.cache_data.exists(old_image_file):
            image_file = old_image_file

        if not self.cache_data.exists(image_file):
            source = item.get_source()
            if source:
                path = self.cache_data.dir + os.sep + image_file
                decoder = timeside.core.get_processor('file_decoder')(source)
                graph = grapher(width=width, height=height)
                (decoder | graph).run()
                graph.watermark('timeside', opacity=.6, margin=(5,5))
                f = open(path, 'w')
                graph.render(output=path)
                f.close()
                self.cache_data.add_file(image_file)

        response = StreamingHttpResponse(self.cache_data.read_stream_bin(image_file), content_type=mime_type)
        return response

    def list_export_extensions(self):
        "Return the recognized item export file extensions, as a list"
        list = []
        for encoder in self.encoders:
            list.append(encoder.file_extension())
        #FIXME: MP4
        list.append('mp4')
        return list

    def item_export(self, request, public_id, extension):
        """Export a given media item in the specified format (OGG, FLAC, ...)"""

        item = MediaItem.objects.get(public_id=public_id)
        public_access = get_item_access(item, request.user)

        if (not public_access == 'full' or not extension in settings.TELEMETA_STREAMING_FORMATS) and \
                    not (request.user.has_perm('telemeta.can_play_all_items') or request.user.is_superuser):
            mess = ugettext('Access not allowed')
            title = 'Item file : ' + public_id + '.' + extension + ' : ' + mess
            description = ugettext('Please login or contact the website administator to get a private access.')
            messages.error(request, title)
            return render(request, 'telemeta/messages.html', {'description' : description})

        #FIXME: MP4 handling in TimeSide
        if 'mp4' in extension:
            mime_type = 'video/mp4'
            video = item.file.path
            response = StreamingHttpResponse(stream_from_file(video), mimetype = mime_type)
            response['Content-Disposition'] = 'attachment'
            return response

        if 'webm' in extension:
            mime_type = 'video/webm'
            video = item.file.path
            response = StreamingHttpResponse(stream_from_file(video), mimetype = mime_type)
            response['Content-Disposition'] = 'attachment'
            return response

        for encoder in self.encoders:
            if encoder.file_extension() == extension:
                break

        if encoder.file_extension() != extension:
            raise Http404('Unknown export file extension: %s' % extension)

        mime_type = encoder.mime_type()
        file = public_id + '.' + encoder.file_extension()
        source = item.get_source()

        flag = MediaItemTranscodingFlag.objects.filter(item=item, mime_type=mime_type)
        if not flag:
            flag = MediaItemTranscodingFlag(item=item, mime_type=mime_type)
            flag.value = False
            flag.save()
        else:
            flag = flag[0]

        format = item.mime_type
        dc_metadata = dublincore.express_item(item).to_list()
        mapping = DublinCoreToFormatMetadata(extension)
        metadata = mapping.get_metadata(dc_metadata)

        if mime_type in format:
            # source > stream
            if not extension in mapping.unavailable_extensions:
                proc = encoder(source, overwrite=True)
                proc.set_metadata(metadata)
                try:
                    #FIXME: should test if metadata writer is available
                    proc.write_metadata()
                except:
                    pass
            response = StreamingHttpResponse(stream_from_file(source), mimetype = mime_type)
        else:
            media = self.cache_export.dir + os.sep + file
            if not self.cache_export.exists(file) or not flag.value:
                # source > encoder > stream
                decoder = timeside.core.get_processor('file_decoder')(source)
                proc = encoder(media, streaming=True, overwrite=True)
                if extension in mapping.unavailable_extensions:
                    metadata=None
                proc.set_metadata(metadata)
                self.cache_export.add_file(file)
                response = StreamingHttpResponse(stream_from_processor(decoder, proc, flag), content_type=mime_type)
            else:
                # cache > stream
                response = StreamingHttpResponse(self.cache_export.read_stream_bin(file), content_type=mime_type)

        response['Content-Disposition'] = 'attachment'
        return response

    def item_playlist(self, request, public_id, template, mimetype):
        try:
            item = MediaItem.objects.get(public_id=public_id)
        except ObjectDoesNotExist:
            raise Http404

        template = loader.get_template(template)
        context = RequestContext(request, {'item': item, 'host': request.META['HTTP_HOST']})
        return HttpResponse(template.render(context), content_type=mimetype)

    @method_decorator(permission_required('telemeta.change_mediaitem'))
    def item_performances_edit(self, request, public_id, template):
        item = MediaItem.objects.get(public_id=public_id)
        PerformanceFormSet = inlineformset_factory(MediaItem, MediaItemPerformance, form=MediaItemPerformanceForm)
        if request.method == 'POST':
            formset = PerformanceFormSet(data=request.POST, instance=item)
            if formset.is_valid():
                formset.save()
                return redirect('telemeta-item-edit', item.public_id)
        else:
            formset = PerformanceFormSet(instance=item)
        return render(request, template, {'item': item, 'formset': formset,})

    @method_decorator(permission_required('telemeta.change_mediaitem'))
    def item_keywords_edit(self, request, public_id, template):
        item = MediaItem.objects.get(public_id=public_id)
        FormSet = inlineformset_factory(MediaItem, MediaItemKeyword)
        if request.method == 'POST':
            formset = FormSet(data=request.POST, instance=item)
            if formset.is_valid():
                formset.save()
                return redirect('telemeta-item-edit', item.public_id)
        else:
            formset = FormSet(instance=item)
        return render(request, template, {'item': item, 'formset': formset,})


class ItemListView(ListView):

    model = MediaItem
    template_name = "telemeta/mediaitem_list.html"
    paginate_by = 20
    queryset = MediaItem.objects.enriched().order_by('code', 'old_code')

    def get_context_data(self, **kwargs):
        context = super(ItemListView, self).get_context_data(**kwargs)
        context['count'] = self.object_list.count()
        return context

class ItemListViewFullAccess(ListView):

    model = MediaItem
    template_name = "telemeta/mediaitem_list.html"
    paginate_by = 20
    queryset = MediaItem.objects.enriched().filter(Q(collection__public_access="full")|Q(public_access="full")).sound().exclude(collection__public_access="none").order_by('code', 'old_code')

    def get_context_data(self, **kwargs):
        context = super(ItemListViewFullAccess, self).get_context_data(**kwargs)
        context['count'] = self.object_list.count()
        return context


class ItemUnpublishedListView(ItemListView):

    queryset = MediaItem.objects.filter(collection__code__contains='_I_').order_by('code', 'old_code')


class ItemPublishedListView(ItemListView):

    queryset = MediaItem.objects.filter(collection__code__contains='_E_').order_by('code', 'old_code')


class ItemSoundListView(ItemListView):

    queryset = MediaItem.objects.sound().order_by('code', 'old_code')


class ItemViewMixin(ItemBaseMixin):

    model = MediaItem
    form_class = MediaItemForm
    inlines = [ItemPerformanceInline, ItemKeywordInline, ItemRelatedInline, ItemIdentifierInline]
    # inlines = [ItemPerformanceInline, ItemKeywordInline, ItemRelatedInline,
    #             ItemFormatInline, ItemIdentifierInline]

    def get_object(self):
        obj = self.model.objects.filter(code=self.kwargs['public_id'])
        if not obj:
            if self.kwargs['public_id'].isdigit():
                try:
                    obj = self.model.objects.get(id=self.kwargs['public_id'])
                except self.model.DoesNotExist:
                    raise Http404
            else:
                raise Http404
        else:
            obj = obj[0]
        return obj


class ItemEditView(ItemViewMixin, UpdateWithInlinesView):

    template_name = 'telemeta/mediaitem_edit.html'

    def get_form_class(self):
        if self.request.user.is_staff:
            return MediaItemForm
        else:
            return RestrictedMediaItemForm

    def forms_valid(self, form, inlines):
        messages.info(self.request, ugettext_lazy("You have successfully updated your item."))
        item = form.save()
        self.code = item.code
        if form.files:
            self.cache_data.delete_item_data(item.code)
            self.cache_export.delete_item_data(item.code)
            flags = MediaItemTranscodingFlag.objects.filter(item=item)
            analyses = MediaItemAnalysis.objects.filter(item=item)
            for flag in flags:
                flag.delete()
            for analysis in analyses:
                analysis.delete()
        item.set_revision(self.request.user)
        return super(ItemEditView, self).forms_valid(form, inlines)

    def get_success_url(self):
        return reverse_lazy('telemeta-item-detail', kwargs={'public_id':self.code})

    def get_context_data(self, **kwargs):
        context = super(ItemEditView, self).get_context_data(**kwargs)
        item = self.get_object()
        context['item'] = item
        context['access'] = get_item_access(item, self.request.user)
        context['previous'], context['next'] = self.item_previous_next(item)
        #FIXME
        context['mime_type'] = 'audio/mp3'
        context['export_formats'] = self.get_export_formats()
        context['visualizers'] = self.get_graphers()
        context['audio_export_enabled'] = self.export_enabled
        context['auto_zoom'] = True
        return context

    @method_decorator(permission_required('telemeta.change_mediaitem'))
    def dispatch(self, *args, **kwargs):
        return super(ItemEditView, self).dispatch(*args, **kwargs)


class ItemAddView(ItemViewMixin, CreateWithInlinesView):

    form_class = MediaItemForm
    template_name = 'telemeta/mediaitem_add.html'

    def get_initial(self):
        item = self.model()
        # new item for a specific collection
        if 'public_id' in self.kwargs:
            public_id = self.kwargs['public_id']
            collections = MediaCollection.objects.filter(code=public_id)
            if collections:
                collection = collections[0]
                item.collection = collection
                items = MediaItem.objects.filter(collection=collection)
                item.code = auto_code(collection)
        return model_to_dict(item)

    def forms_valid(self, form, inlines):
        messages.info(self.request, ugettext_lazy("You have successfully added your item."))
        obj = form.save()
        obj.set_revision(self.request.user)
        return super(ItemAddView, self).forms_valid(form, inlines)

    def get_success_url(self):
        return reverse_lazy('telemeta-item-detail', kwargs={'public_id':self.object.code})

    @method_decorator(permission_required('telemeta.add_mediaitem'))
    def dispatch(self, *args, **kwargs):
        return super(ItemAddView, self).dispatch(*args, **kwargs)


class ItemCopyView(ItemAddView):

    form_class = MediaItemForm
    template_name = 'telemeta/mediaitem_edit.html'

    def get_initial(self):
         return model_to_dict(self.get_object())

    def forms_valid(self, form, inlines):
        messages.info(self.request, ugettext_lazy("You have successfully updated your item."))
        item = form.save()
        item.set_revision(self.request.user)
        if not MediaItemPerformance.objects.filter(media_item=item):
            for performance in MediaItemPerformance.objects.filter(media_item=self.get_object()):
                performance.pk = None
                performance.id = None
                performance.media_item = item
                performance.save()
        if not MediaItemKeyword.objects.filter(item=item):
            for keyword in MediaItemKeyword.objects.filter(item=self.get_object()):
                keyword.pk = None
                keyword.id = None
                keyword.item = item
                keyword.save()
        return super(ItemCopyView, self).forms_valid(form, inlines)

    def get_success_url(self):
        return reverse_lazy('telemeta-item-detail', kwargs={'public_id':self.object.code})

    def get_context_data(self, **kwargs):
        context = super(ItemCopyView, self).get_context_data(**kwargs)
        item = self.get_object()
        context['item'] = item
        context['access'] = get_item_access(item, self.request.user)
        context['previous'], context['next'] = self.item_previous_next(item)
        #FIXME
        context['mime_type'] = 'audio/mp3'
        context['export_formats'] = self.get_export_formats()
        context['visualizers'] = self.get_graphers()
        context['audio_export_enabled'] = self.export_enabled
        context['auto_zoom'] = True
        return context

    @method_decorator(permission_required('telemeta.add_mediaitem'))
    def dispatch(self, *args, **kwargs):
        return super(ItemCopyView, self).dispatch(*args, **kwargs)


class ItemDetailView(ItemViewMixin, DetailView):

    template_name = 'telemeta/mediaitem_detail.html'

    def item_analyze(self, item):
        analyses = item.analysis.all()
        mime_type = ''

        if analyses:
            for analysis in analyses:
                if not item.approx_duration and analysis.analyzer_id == 'duration':
                    value = analysis.value
                    time = value.split(':')
                    time[2] = time[2].split('.')[0]
                    time = ':'.join(time)
                    item.approx_duration = time
                    item.save()
                if analysis.analyzer_id == 'mime_type':
                    mime_type = analysis.value
        else:
            analyzers = []
            analyzers_sub = []
            graphers_sub = []

            source = item.get_source()
            if source:
                decoder  = timeside.core.get_processor('file_decoder')(source)
                pipe = decoder

                for analyzer in self.value_analyzers:
                    subpipe = analyzer()
                    analyzers_sub.append(subpipe)
                    pipe = pipe | subpipe

                default_grapher = self.get_grapher(self.default_grapher_id)
                for size in self.default_grapher_sizes:
                    width = size.split('x')[0]
                    height = size.split('x')[1]
                    image_file = '.'.join([item.public_id, self.default_grapher_id, size.replace('x', '_'), 'png'])
                    path = self.cache_data.dir + os.sep + image_file
                    graph = default_grapher(width = int(width), height = int(height))
                    graphers_sub.append({'graph' : graph, 'path': path})
                    pipe = pipe | graph

                pipe.run()

                for grapher in graphers_sub:
                    grapher['graph'].watermark('timeside', opacity=.6, margin=(5,5))
                    f = open(grapher['path'], 'w')
                    grapher['graph'].render(grapher['path'])
                    f.close()

                if os.path.exists(source):
                    mime_type = mimetypes.guess_type(source)[0]
                    analysis = MediaItemAnalysis(item=item, name='MIME type',
                                             analyzer_id='mime_type', unit='', value=mime_type)
                    analysis.save()
                    analysis = MediaItemAnalysis(item=item, name='Size',
                                             analyzer_id='size', unit='', value=item.size())
                    analysis.save()

                analysis = MediaItemAnalysis(item=item, name='Channels',
                                             analyzer_id='channels',
                                             unit='', value=decoder.input_channels)
                analysis.save()
                analysis = MediaItemAnalysis(item=item, name='Samplerate',
                                             analyzer_id='samplerate', unit='Hz',
                                             value=unicode(decoder.input_samplerate))
                analysis.save()
                analysis = MediaItemAnalysis(item=item, name='Resolution',
                                             analyzer_id='resolution', unit='bits',
                                             value=unicode(decoder.input_width))
                analysis.save()
                analysis = MediaItemAnalysis(item=item, name='Duration',
                                             analyzer_id='duration', unit='s',
                                             value=unicode(datetime.timedelta(0,decoder.input_duration)))
                analysis.save()

                for analyzer in analyzers_sub:
                    for key in analyzer.results.keys():
                        result = analyzer.results[key]
                        value = result.data_object.value
                        if value.shape[0] == 1:
                            value = value[0]
                        analysis = MediaItemAnalysis(item=item, name=result.name,
                                analyzer_id=result.id, unit=result.unit, value = unicode(value))
                        analysis.save()

#                FIXME: parse tags on first load
#                tags = decoder.tags

        self.mime_type = mime_type

    def get_context_data(self, **kwargs):
        context = super(ItemDetailView, self).get_context_data(**kwargs)

        public_id = get_kwargs_or_none('public_id', self.kwargs)
        marker_id = get_kwargs_or_none('marker_id', self.kwargs)
        width = get_kwargs_or_none('width', self.kwargs)
        height = get_kwargs_or_none('height', self.kwargs)

        # get item with one of its given marker_id
        if not public_id and marker_id:
            marker = MediaItemMarker.objects.get(public_id=marker_id)
            item_id = marker.item_id
            item = MediaItem.objects.get(id=item_id)
        else:
            item = self.get_object()

        access = get_item_access(item, self.request.user)

        previous, next = self.item_previous_next(item)

        self.item_analyze(item)

        #FIXME: use mimetypes.guess_type
        if 'quicktime' in self.mime_type:
            self.mime_type = 'video/mp4'

        playlists = get_playlists_names(self.request)
        related_media = MediaItemRelated.objects.filter(item=item)
        check_related_media(related_media)
        revisions = Revision.objects.filter(element_type='item', element_id=item.id).order_by('-time')
        if revisions:
            last_revision = revisions[0]
        else:
            last_revision = None

        item_format = ''
        if Format.objects.filter(item=item):
            item_format = item.format.get()

        context['item'] = item
        context['export_formats'] = self.get_export_formats()
        context['visualizers'] = self.get_graphers()
        context['auto_zoom'] = self.auto_zoom
        context['audio_export_enabled'] = self.export_enabled
        context['previous'] = previous
        context['next'] = next
        context['marker'] = marker_id
        context['playlists'] = playlists
        context['access'] = access
        context['width'] = width
        context['height'] = height
        context['related_media'] = related_media
        context['mime_type'] = self.mime_type
        context['last_revision'] = last_revision
        context['format'] = item_format
        context['private_extra_types'] = private_extra_types.values()
        context['site'] = 'http://' + Site.objects.all()[0].name
        return context


class DublinCoreToFormatMetadata(object):
    """a mapping class to get item DublinCore metadata dictionaries
    in various audio metadata format (MP3, OGG, etc...)"""

    #FIXME: should be given by timeside
    unavailable_extensions = ['wav', 'aiff', 'aif', 'flac', 'webm']

    metadata_mapping = {
                    'mp3' : {
                         'title': 'TIT2', #title2
                         'creator': 'TCOM', #composer
                         'creator': 'TPE1', #lead
                         'identifier': 'UFID', #unique ID
                         'relation': 'TALB', #album
                         'type': 'TCON', #genre
                         'publisher': 'TPUB', #publisher
                         'date': 'TDRC', #year
#                         'coverage': 'COMM',  #comment
                         },
                    'ogg': {
                        'creator': 'artist',
                        'relation': 'album',
                        'all': 'all',
                       },
                    'flac': {
                        'creator': 'artist',
                        'relation': 'album',
                        'all': 'all',
                       },
                    'wav': {
                        'creator': 'artist',
                        'relation': 'album',
                        'all': 'all',
                       },
                    'webm': {
                        'creator': 'artist',
                        'relation': 'album',
                        'all': 'all',
                       },
                    }

    def __init__(self, format):
        self.format = format

    def get_metadata(self, dc_metadata):
        mapp = self.metadata_mapping[self.format]
        metadata = {}
        keys_done = []
        for data in dc_metadata:
            key = data[0]
            value = data[1].encode('utf-8')
            if value:
                if key == 'date':
                    value = value.split(';')[0].split('=')
                    if len(value) > 1:
                        value  = value[1]
                        value = value.split('-')[0]
                    else:
                        value = value[0].split('-')[0]
                if key in mapp:
                    metadata[mapp[key]] = value.decode('utf-8')
                elif 'all' in mapp.keys():
                    metadata[key] = value.decode('utf-8')
                keys_done.append(key)
        return metadata


class ItemMarkerJsonView(View):

    model = MediaItem

    def get(self, request, *args, **kwargs):
        code = self.kwargs['public_id']
        marker_view = MarkerView()
        item = MediaItem.objects.get(code=code)
        markers = marker_view.get_markers(item.id)
        if markers:
            data = json.dumps(markers)
        else:
            data = ''
        response = HttpResponse(data, content_type='application/json')
        response['Content-Disposition'] = "attachment; filename=%s.%s" % \
                                             (item.code, 'json')
        return response


class ItemPlayerDefaultView(ItemDetailView):

    template_name = 'telemeta/mediaitem_player.html'


class ItemDetailDCView(ItemDetailView):

    template_name = 'telemeta/mediaitem_detail_dc.html'


class ItemVideoPlayerView(ItemDetailView):

    template_name = 'telemeta/mediaitem_video_player.html'



# -*- coding: utf-8 -*-
# Copyright (C) 2010-2012 Parisson SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#          Guillaume Pellerin <yomguy@parisson.com>

import re
import mimetypes
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError
from telemeta.models.core import *
from telemeta.util.unaccent import unaccent_icmp
from telemeta.models.enum import *
from telemeta.models.language import *
from django.db import models


class Format(ModelCore):
    """ Physical format object as proposed by the LAM"""

    element_type          = 'format'

    item                  = ForeignKey('MediaItem', related_name="format", verbose_name = _("item"),
                                       blank=True, null=True, on_delete=models.SET_NULL)
    physical_format       = WeakForeignKey(PhysicalFormat, related_name="format",
                                     verbose_name = _("physical format"))
    original_code         = CharField(_('original code'))
    original_number       = CharField(_('original number'))
    original_status       = CharField(_('original status'))
    original_state        = TextField(_('technical properties / conservation state'))
    original_comments     = TextField(_('comments / notes'))
    original_location     = ForeignKey('Location', related_name="format",
                                       verbose_name = _("original location"),
                                       blank=True, null=True, on_delete=models.SET_NULL)
    original_channels     = WeakForeignKey(NumberOfChannels, related_name="format",
                                        verbose_name = _("number of channels"))
    original_audio_quality = TextField(_('audio quality'))
    recording_system      = CharField(_('recording system'))

    # Tapes
    tape_wheel_diameter = WeakForeignKey(TapeWheelDiameter, related_name="format",
                                        verbose_name = _("tape wheel diameter (cm)"))
    tape_thickness      = CharField(_('tape thickness (um)'))
    tape_speed          = WeakForeignKey(TapeSpeed, related_name="format",
                                        verbose_name = _("tape speed (cm/s)"))
    tape_vendor         = WeakForeignKey(TapeVendor, related_name="format",
                                        verbose_name = _("tape vendor"))
    tape_reference      = CharField(_('tape reference'))
    sticker_presence    = BooleanField(_('sticker presence'))

    class Meta(MetaCore):
        db_table = 'media_formats'
        verbose_name = _('format')

    def __unicode__(self):
        if self.physical_format:
            return ' - '.join([self.physical_format.value, self.original_code,
                               self.item.public_id])
        else:
            return 'Unknown'

    @property
    def public_id(self):
        return self.original_code



import datetime
from django.db import models


# Ghetto app!
class Note(models.Model):
    title = models.CharField(max_length=128)
    content = models.TextField()
    author = models.CharField(max_length=64)
    created = models.DateTimeField(default=datetime.datetime.now)
    
    def __unicode__(self):
        return self.title



from optparse import make_option
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from telemeta.models import Location
from telemeta.util.unaccent import unaccent
import logging
import codecs

class Command(BaseCommand):
    help = "Geocode Telemeta countries from a local Geonames data file"
    args = "path to geoname's allCountries.txt"

    def handle(self, datafile=None, *args, **options):

        if not datafile:
            raise CommandError("Please provide the %s" % self.args)

        try:
            datafile = codecs.open(datafile, 'r', 'utf-8')
        except IOError:
            raise CommandError("Unable to open %s" % datafile)
        
        locations = {}
        for l in Location.objects.all().current().filter(type=Location.COUNTRY):
            locations[l] = [a.alias for a in l.aliases.all()]

        i = 0
        geocoded = 0
        total = len(locations)
        found_by_alias = {}
        for line in datafile:
            (geonameid, name, asciiname, alternatenames, latitude, longitude, feature_class,
             feature_code, country_code, cc2, admin1_code, admin2_code, admin3_code,
             admin4_code, population, elevation, gtopo30, timezone, modification_date) = line.strip().split("\t")
           
            if feature_code[0:3] == 'PCL':
                names = [asciiname.lower()]
                if alternatenames:
                    names.extend([unaccent(n).lower() for n in alternatenames.split(',')])

                found = []
                for l in locations:
                    if unaccent(l.name).lower() in names:
                        l.latitude = float(latitude)
                        l.longitude = float(longitude)
                        l.save()
                        geocoded += 1
                        found.append(l)
                    else:
                        for a in locations[l]:
                            if unaccent(a).lower() in names:
                                found_by_alias[l] = float(latitude), float(longitude)
                                break
                            

                for l in found:
                    locations.pop(l)

            i += 1

            if i % 200000 == 0:
                print "Geocoded %d (%d by alias) out of %d countries (parsed %d geonames)" % (geocoded, len(found_by_alias), total, i)

            if total == geocoded:
                break

        for l in locations:
            if found_by_alias.has_key(l):
                l.latitude, l.longitude = found_by_alias[l]
                l.save()
                geocoded += 1

        print "Done. Geocoded %d out of %d countries (parsed %d geonames)" % (geocoded, total, i)
        datafile.close()                




from django.conf import settings
import re
import os
import telemeta
import mimetypes

PAGES_ROOT = os.path.join(os.path.dirname(telemeta.__file__), 'pages')

class PageTextContent(object):
    def __init__(self, filename, path):
        self.filename = filename
        self.path = path

    def __iter__(self):
        file = open(self.filename, 'r')
        for line in file:
            yield line.rstrip('\r\n')
        file.close()

    def __unicode__(self):
        file = open(self.filename, 'r')
        data = file.read()
        file.close()
        return data

class PageAttachment(object):
    def __init__(self, filename, path):
        self.filename = filename
        self.path     = path

    def mimetype(self):
        type, encoding = mimetypes.guess_type(self.filename)
        return type

    def __iter__(self):
        file = open(self.filename, 'rb')
        buffer_size = 0x10000
        while True:
            chunk = file.read(buffer_size)
            yield chunk
            if len(chunk) < buffer_size:
                break

        file.close()

def language_code(request=None):
    code = (request and getattr(request, 'LANGUAGE_CODE', None)) or settings.LANGUAGE_CODE
    cut = re.split('[_-]', code)
    code = cut[0]
    return code.lower()

def project_dir():
    try:
        import sandbox.settings as settings_mod
    except:
        import settings as settings_mod
    if '__init__.py' in settings_mod.__file__:
        p = os.path.dirname(settings_mod.__file__)
    else:
        p = settings_mod.__file__
    project_directory, settings_filename = os.path.split(p)
    if project_directory == os.curdir or not project_directory:
        project_directory = os.getcwd()

    return project_directory

def resolve_page_file(root, relative_path, ignore_slash_issue=False):
    root = os.path.realpath(root)
    filename = None
    current = root
    is_attachment = False
    for node in relative_path.split('/'):
        if not node:
            continue
        current = os.path.join(current, node)
        rst = current + '.rst'
        if os.path.isfile(rst):
            filename = rst
            break
        elif os.path.isfile(current):
            filename      = current
            is_attachment = True
        elif not os.path.isdir(current):
            break

    if not filename and os.path.isdir(current):
        rst = os.path.join(current, 'index.rst')
        if os.path.isfile(rst):
            if not ignore_slash_issue and relative_path[-1:] != '/':
                raise MalformedPagePath("The relative page os.path must end with a slash when "
                                        "resolving an implicit directory index")
            filename = rst

    if filename:
        filename = os.path.realpath(filename)
        if filename.index(root) != 0:
            filename = None

    if filename:
        if is_attachment:
            return PageAttachment(filename, relative_path)
        else:
            return PageTextContent(filename, relative_path)

    return None

def get_page_content(request, relative_path, ignore_slash_issue=False):
    lang = language_code(request)
    userroot = os.path.join(project_dir(), 'telemeta-pages')
    rootlist = [os.path.join(userroot, lang), os.path.join(userroot, 'en'),
                os.path.join(PAGES_ROOT, lang), os.path.join(PAGES_ROOT, 'en')]
    for root in rootlist:
        content = resolve_page_file(root, relative_path, ignore_slash_issue=ignore_slash_issue)
        if content:
            return content

    return None

class MalformedPagePath(Exception):
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2007 Samalyse SARL

# This file is part of Telemeta.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Olivier Guilyardi <olivier@samalyse.com>

from telemeta.models.core import *
from telemeta.models.item import *
from telemeta.models.collection import *
from django.contrib.sites.models import Site
from django.conf import settings


class Resource(object):
    "Represent a Dublin Core resource"

    def __init__(self, *args):
        self.elements = []
        self.add(*args)

    def flatten(self):
        """Convert the resource to a dictionary with element names as keys.

        Warnings:
        - refinements are lost during this conversion,
        - if there are several occurences of the same element, only the first is
        used, others are lost.
        - all values are converted to strings
        """
        result = {}
        for element in self.elements:
            if not result.has_key(element.name):
                result[element.name] = unicode(element.value)
        return result

    def to_list(self):
        """Convert the resource to unqualified dublin core, as a list of the form:
           [(key, value), ...]"""
        result = []
        for element in self.elements:
            if isinstance(element.value, str):
                value = element.value.decode('utf8')
            else:
                value = element.value
            result.append((element.name, unicode(value)))
        return result

    def add(self, *elements):
        for e in elements:
            if isinstance(e, Element):
                if not e in self.elements:
                    self.elements.append(e)
            else:
                try:
                    iter(e)
                except TypeError:
                    raise Exception("add() only accepts elements or sequences of elements")

                self.add(*e)

    def __unicode__(self):
        dump = u''
        for e in self.elements:
            key = unicode(e.name)
            if e.refinement:
                key += u'.' + unicode(e.refinement)
            dump += u'%s:\t%s\n' % (key, unicode(e.value))
        return dump


class Element(object):
    "Represent a Dublin Core element"

    def __init__(self, name, value=None, refinement=None, related=None):
        self.name = name
        self.value = value
        self.refinement = refinement
        self.related = related

    def __eq__(self, other):
        return self.name == other.name and self.value == other.value and self.refinement == self.refinement

    def __ne__(self, other):
        return not (self == other)

    @staticmethod
    def multiple(name, values, refinement=None):
        elements = []
        if values:
            for v in values:
                elements.append(Element(name, v, refinement))
        return elements

class Date(Element):
    "Dublin Core date element formatted according to W3C-DTF or DCMI Period"

    def __init__(self, start, end=None, refinement=None):
        value = ''
        if start:
            value = start
        elif end:
            value = end
        else:
            value = ''
        if isinstance(value, long):
            # start is a year
            value = unicode(value) + '-01-01T00:00:00Z'
        elif value:
            value = value.strftime('%Y-%m-%dT%H:%M:%SZ')

        super(Date, self).__init__('date', value, refinement)

def media_access_rights(media):
    if media.public_access == 'full':
        return 'public'
    if media.public_access == 'metadata':
        return 'restricted'
    return 'private'

def media_identifier(media):
    sites = Site.objects.all()
    domain = sites[0].domain
    return 'http://' + domain + '/' + media.element_type + 's/' + unicode(media.id)

def media_generic_identifier(media):
    sites = Site.objects.all()
    domain = sites[0].domain
    return 'http://' + domain + '/' + media.element_type + '/' + unicode(media.id)

def express_collection(collection):
    "Express a collection as a Dublin Core resource"

    if collection.collector:
        creator = (Element('creator', collection.collector),
                   Element('contributor', collection.creator))
    else:
        creator = Element('creator', collection.creator)

    duration = max(collection.approx_duration, collection.computed_duration())
    parts = []
    for item in collection.items.all():
        id = media_identifier(item)
        if id:
            parts.append(Element('relation', id, 'hasPart', item))

    titles = [Element('title', collection.title),]
    if collection.alt_title:
        titles.append(Element('title', collection.alt_title))

    dates = []
    if collection.recorded_from_year or collection.recorded_to_year:
        dates.append(Date(collection.recorded_from_year, collection.recorded_to_year, refinement='created'))
    if collection.year_published:
        dates.append(Date(collection.year_published, refinement='issued'))

    resource = Resource(
        Element('identifier',       media_identifier(collection), related=collection),
        Element('identifier',       collection.public_id, related=collection),
        Element('type',             'Collection'),
        titles,
        creator,
        Element('contributor',      collection.metadata_author),
        Element.multiple('subject', settings.TELEMETA_SUBJECTS),
        Element('publisher',        collection.publisher),
        Element('publisher',        settings.TELEMETA_ORGANIZATION),
        dates,
        Element('rights',           collection.legal_rights, 'license'),
        Element('rights',           media_access_rights(collection), 'accessRights'),
        Element('format',           duration, 'extent'),
        Element('format',           collection.physical_format, 'medium'),
        parts
    )

    return resource

def express_item(item):
    "Express a media item as a Dublin Core resource"

    if item.collector:
        creator = (Element('creator', item.collector),
                   Element('contributor', item.collection.creator))
    elif item.collection.collector:
        creator = (Element('creator', item.collection.collector),
                   Element('contributor', item.collection.creator))
    else:
        creator = Element('creator', item.collection.creator)

    dates = []
    if item.recorded_from_date:
        dates.append(Date(item.recorded_from_date, item.recorded_to_date, refinement='created'))
    elif item.collection.recorded_from_year or item.collection.recorded_to_year:
        dates.append(Date(item.collection.recorded_from_year, item.collection.recorded_to_year, refinement='created'))
    if item.collection.year_published:
        dates.append(Date(item.collection.year_published, refinement='issued'))

    titles = []
    if item.title:
        title = item.title
    else:
        title = item.collection.title
        if item.track:
            title += u' - ' + item.track
    titles.append(Element('title', item.title))

    if item.alt_title:
        titles.append(Element('title', item.alt_title))

    try:
        analysis = MediaItemAnalysis.objects.get(item=item, analyzer_id='mime_type')
        mime_type = analysis.value
    except:
        mime_type = ''

    resource = Resource(
        Element('identifier',       media_identifier(item), related=item),
        Element('identifier',       item.public_id, related=item),
        Element('type',             'Sound'),
        titles,
        creator,
        Element('contributor',      item.collection.metadata_author),
        Element.multiple('subject', settings.TELEMETA_SUBJECTS),
        Element.multiple('subject', item.keywords()),
        Element('description',      item.context_comment, 'abstract'),
        Element('publisher',        item.collection.publisher),
        Element('publisher',        settings.TELEMETA_ORGANIZATION),
        dates,
        Element.multiple('coverage', item.location and item.location.listnames(), 'spatial'),
        Element('coverage',         item.location_comment, 'spatial'),
        Element('rights',           item.collection.legal_rights, 'license'),
        Element('rights',           media_access_rights(item.collection), 'accessRights'),
        Element('format',           max(item.approx_duration, item.computed_duration()), 'extent'),
        Element('format',           item.collection.physical_format, 'medium'),
        Element('format',           mime_type, 'MIME type'),
        Element('relation',         media_identifier(item.collection), 'isPartOf', item.collection)
    )

    return resource

def express_generic_resource(resource):
    "Express a media item as a Dublin Core resource"

    parts = []
    for child in resource.children.all():
        id = media_generic_identifier(child)
        if id:
            parts.append(Element('relation', id, 'hasPart', child))

    r = Resource(
        Element('identifier',       media_generic_identifier(resource), related=resource),
        Element('identifier',       resource.public_id, related=resource),
        Element('type',             resource.element_type),
        Element('title',            resource.title),
        Element('description',      resource.description),
        Element('publisher',        settings.TELEMETA_ORGANIZATION),
        parts
    )

    return r


def express_resource(res):
    if isinstance(res, MediaItem):
        return express_item(res)
    elif isinstance(res, MediaCollection):
        return express_collection(res)

    raise Exception("Invalid resource type")

def lookup_resource(media_id):
    try:
        id = media_id.split(':')
        type = id[-2]
        code = id[-1]
    except ValueError:
        raise MalformedMediaIdentifier("Media identifier must be in type:code format")

    if (type == 'collection') or (type == 'collections'):
        try:
            return MediaCollection.objects.get(code=code)
        except MediaCollection.DoesNotExist:
            return None
    elif (type == 'item') or (type == 'items'):
        try:
            return MediaItem.objects.get(code=code)
        except MediaItem.DoesNotExist:
            try:
                return MediaItem.objects.get(old_code=code)
            except MediaItem.DoesNotExist:
                try:
                    return MediaItem.objects.get(id=code)
                except MediaItem.DoesNotExist:
                    return None
    else:
        raise MalformedMediaIdentifier("No such type in media identifier: " + type)

class MalformedMediaIdentifier(Exception):
    pass

